# prosomain.praat -- include file for prosogram.praat
# Author: Piet Mertens
# Last modification: 2015-04-29
# Requires Praat 5.4 or higher.

# Modify the following lines to match the path of the application on your computer.
path_ghostscript$ = "c:\Program Files\gs\gs9.16\bin\gswin64c.exe"	; typical path on Windows-8 64 bit
path_nconvert$ = "c:\Program Files\XnView\nconvert"

# Don't change the following lines:
path_ghostscript$ = replace$ (path_ghostscript$, "\Program Files\", "\Progra~1\", 1)
path_nconvert$ = replace$ (path_nconvert$, "\Program Files\", "\Progra~1\", 1)

version$ = "Prosogram v2.11"

font_family$ = "Times"	; Font used in plot; else use "Helvetica"
viewport_width = 7.5	; width of viewport used for prosogram (inches)
clip_to_Y_range = 0		; Clip stylisation to Y range
greyscale = 0			; Use greyscale instead of colors

boundary_annotation = 0		; automatic detection of prosodic boundaries
contour_annotation = 0		; automatic annotation of pitch movements and pitch levels
stress_annotation = 0		; automatic detection of stress
include_loudness = 0		; 1 when loudness is included in spreadsheet
plot_pauses = 1				;
show_pseudosyllables = 0	; show pseudosyllables in textgrid
show_harmonicity = 0		;
show_localrate = 0			; used in gr_display_prosogram
single_fname_graphics_output = 0; when true, all graphic files will numbered using the same basename
mindur_pause_anno = 0.2		; min duration for pause in annotation
prefilterHP100 = 0			; Apply HP filtering to avoid problems with low quality audio recordings
prefilterLP = 0				; Apply LP filtering when fricatives are too strong (low quality audio recording)
avoid_insuff_memory = 0		; Used in calculation of intensity of BP filtered signal
use_duration_model = 0		;
; duration_model_filename$ = "db_phon_dur_rhapsodie.txt"
; duration_model_filename$ = "db_phon_dur_cprom.txt"
duration_model_filename$ = ""
show_elasticity = 0			; used in gr_display_prosogram
calc_prominence = 0			; compute prominence measures
show_prominence = 0			;

corpus$ = ""			; "" = default corpus conventions; known corpora : rhapsodie, cprom, project_frfc, RUHRCAT, Prsir, gvlex

include prosoplot.praat
include segment.praat
include stylize.praat
include util.praat

;if (boundary_annotation or contour_annotation)
;call logging_start "" _log.txt
;include rules.praat
;include prominence.praat
;include duration.praat
;include elasticity.praat
;endif



; types of tasks
   task_calc_pitch = 1
   task_calc_loudness = 10
   task_calc_bp_intensity = 13
   task_pitch_plot = 2
   task_segmentation = 3	; automatic segmentation into syllabic nuclei without phonetic alignment annotation
   task_prosogram = 4
   task_interactive = 5
   task_annotation = 6		; Draw annotation only
   task_profile = 7
   task_validate_tiers = 12
   task_autoanno = 9		; only when using form
   ;task_boundaries = 8
   ;task_contours = 9

; types of segmentation
   segm_vnucl = 1
   segm_extern = 2
   segm_aloudness = 3
   segm_anucl = 4
   segm_asyll = 5
   segm_msyllvow = 6
   segm_msyllpeak = 7
   segm_mrhyme = 8		; syllable rime using phoneme and syllable annotation
   segm_voiced = 9		; voiced portions


if (not variableExists ("batch_mode"))	; prosomain script started directly, not from prosogram.praat
   batch_mode = 1
   # variables initialized before batch functions
   save_intermediate_data = 0
   output_filename$ = ""
   clip_level = 0
   scale_signal_amplitude = 0
   volatile = 0
   view = 4
   show_pitchrange = 0
endif

clearinfo			; clear the info window


procedure main
   if (praatVersion < 5400)
      call fatal_error Requires Praat version 5.4 or higher. Please upgrade your Praat version. 
   endif

   call process_form
   call process_multiple_input_files task "'input_files$'" anal_t1 anal_t2

   call msg Ready
endproc


procedure process_multiple_input_files .task .filespec$ pmf_t1 pmf_t2
# <pmf_t1>	starttime as used in call to multiple file batch processing
   task = .task		; needed in batch_mode, because global variable <task> is defined in process_form
   if (index (.filespec$, "*"))		; wildcard found
      filelistID = Create Strings as file list... filelist '.filespec$'
      nrofFiles = Get number of strings
      .wildcard_found = 1
   else
      .wildcard_found = 0
      nrofFiles = 0
      if (fileReadable (.filespec$))
         nrofFiles = 1
      endif
   endif
   if (nrofFiles == 0)
      call fatal_error No input files found for <'.filespec$'>
   endif
   ; filename returned by 'Get string' does not include path
   call fname_parts '.filespec$'
   indir$ = result4$
   call task_flags
   call initialization_multiple_files
   if (task == task_interactive  and nrofFiles > 1)
      call msg Interactive mode uses a single speech file, not many.
      nrofFiles = 1
   endif
   for iFile to nrofFiles
      if (not .wildcard_found)
         .filespec$ = replace_regex$ (.filespec$, "\*", "",1)	; remove '*'
         call fname_parts '.filespec$'
         fname$ = result1$
      else       
         select Strings filelist
         fname$ = Get string... iFile
         ; filename returned by 'Get string' does not include path
      endif
      call process_one_input_file pmf_t1 pmf_t2 'indir$''fname$'
   endfor
   call cleanup_global
endproc


procedure process_one_input_file anal_t1 anal_t2 .fullname$
   call initialization_per_file
   call construct_filenames '.fullname$'
   call msg 'newline$'Processing input file 'fname$'...

   if (not fileReadable (segfile$))
      call error_msg Could not find annotation file <'segfile$'>
      goto file_done
   endif

   if (task == task_interactive)	; Read segmentation_name from nucleus file
      call peek_settings
      @gr_start_demowin: basename$
      @gr_printline: "Processing input file 'fname$'..."
   endif

   if (needs_segm_type)
      if (task == task_segmentation)
         segmentation_name$ = "asyll"
      endif
      call map_segmentation_type 'segmentation_name$'
   endif

   if (task == task_validate_tiers)
      call validate_syllable_tier 'segfile$'
      goto file_done
   elsif (task = task_calc_pitch)
      call calculate_pitch_int
      removeObject: pitchID
      pitch_available = 0
      goto file_done
   elsif (task = task_calc_bp_intensity)
      call calculate_intensity_bp_filtered
      removeObject (intbpID)
      intbp_available = 0
      goto file_done
   elsif (task = task_calc_loudness)
      call calculate_loudness
      removeObject (loudnessID)
      loudness_available = 0
      goto file_done
   endif

   call peek_signal
   anal_t1 = max(anal_t1, signal_start)
   if (anal_t2 == 0)
      anal_t2 = signal_finish
   elsif (anal_t2 > 0)
      anal_t2 = min(anal_t2, signal_finish)
   endif
   if (anal_t1 >= signal_finish)
      call fatal_error The start time you supplied ('anal_t1:3') is outside the time range of the sound ('signal_start:3' - 'signal_finish:3')
   endif
   if (anal_t1 >= anal_t2)
      call fatal_error The start time you supplied ('anal_t1:3') is larger than the used end time ('anal_t2:3')
   endif

   if (task == task_prosogram or task == task_interactive)
      if (anal_t1 == signal_start and (abs(anal_t2 - signal_finish) < time_step))
         needs_pitchrange = 1	; will calculate pitchrange (for prosodic profile of for interactive mode)
      endif
   endif

   .minutes = (anal_t2 - anal_t1) div 60
   .sec = (anal_t2 - anal_t1) mod 60
   call msg Analysis time range: 'anal_t1:3' - 'anal_t2:3' s ('.minutes' min, '.sec:3' s)

   if (need_parameters)
      @gr_printline: "Reading parameter files..."
      call read_parameter_files
      @gr_printline: "Calculating parameters..."
      call calculate_parameters
   endif

   if (task != task_interactive and signal_available)	; Free memory if possible
      removeObject: soundID
      signal_available = 0
   endif

   call read_annotation_file
   if (segfile_available) ; apply conversion to segmentation textgrid befor making textgrid for plotting
      call corpus_conversion 1 "segmentationID" "'corpus$'" 'contourfile$'
   endif

   if (task != task_calc_pitch and task != task_calc_bp_intensity and task != task_segmentation and task != task_validate_tiers and task != task_profile)
      call msg Preparing TextGrid plotted in prosograms...
      call prepare_plotted_textgrid 'tiers_to_show$'
   endif

   if (needs_segm_tg)	; apply conversion to segmentation textgrid after making textgrid for plotting
      call corpus_conversion 2 "segmentationID" "'corpus$'" 'contourfile$'
   endif

   if (need_stylization or task == task_segmentation)
      if (contour_annotation or needs_pitchrange)		; analyse entire signal
         t1s = signal_start
         t2s = signal_finish
      else												; analyse only part of the speech signal
         t1s = anal_t1
         t2s = anal_t2
      endif
      call msg Segmentation into syllabic nuclei. (Method='segmentation_name$', Time range='t1s:3'-'t2s:3')
      call segmentation segm_type
      ; or reads segmentation from file
      ; also prepares textgrid "nucleiID" with intermediate data 
   endif

   if (task == task_segmentation)
      select nucleiID
      tmp1 = Extract tier... nucleus_tier
      select nucleiID
      tmp2 = Extract tier... syllable_tier
      select tmp1
      plus tmp2
      tmp3 = Into TextGrid
      call nucleus_tier_postproc tmp3 1 2
      Write to text file... 'autosegfile$'
      removeObject (tmp1, tmp2, tmp3)
      call msg Segmentation written to 'autosegfile$'
      goto file_done
   endif

   if (need_stylization)
      if (nrof_nuclei_analysed < 1)
         call error_msg No syllabic nuclei found in speech signal.
         call error_msg If you are using segmentation from annotation tier (phon... or syll...), check tier name and tier content.
         goto file_done
      endif
      call create_table_of_nuclei
   endif

; printline prosomain: reuse_nucl='reuse_nucl'
   if (task == task_interactive and reuse_nucl)
    ; stylization cannot be used by boundary_annotation, because procedure stylize computes data in nucldatID
         if (not fileReadable (stylfile$))
            @gr_printline: "Cannot open stylization file 'stylfile$'..."
            call msg Cannot open stylization file 'stylfile$'...
         elsif (not fileReadable (sheetfile$))
            @gr_printline: "Cannot open prosodic features from file 'sheetfile$'..."
            call msg Cannot open prosodic features from file 'sheetfile$'...
         else
            @gr_printline: "Loading stylization from file 'stylfile$'..."
            call msg Loading stylization from file 'stylfile$'...
            stylID = Read from file... 'stylfile$'
            stylization_available = 1
            reuse_styl = 1
            removeObject: nucldatID
            @gr_printline: "Loading prosodic features from file 'sheetfile$'..."
            call msg Loaded prosodic features from file 'sheetfile$'...
            nucldatID = Read TableOfReal from headerless spreadsheet file... 'sheetfile$'
         endif
   endif
; printline prosomain: need_stylization='need_stylization' stylization_available='stylization_available'
   if (need_stylization)
      if (not stylization_available)
         # Create pitch tier object for stylization
         if (adaptive_glissando)
            s$ = "G(adapt)='glissando_low'-'glissando'/T^2"
         else
            s$ = "G='glissando'/T^2"
         endif
         s$ = "'segmentation_name$', " + s$ + ", DG='diffgt', dmin='mindur_ts:3'"
         @gr_printline: "Calculating stylization... ('s$')"
         stylID = Create PitchTier... stylization signal_start signal_finish
         call stylize_nuclei t1s t2s
         stylization_available = 1
      endif
      call speaker_info_get
      if (needs_pitchrange)
         if (variableExists ("corpus$") and corpus$ = "Proust" and variableExists ("corpus_speaker_range_1$"))
            call msg Reading pitch range from corpus_speaker_range...
            speaker_range_1$ = corpus_speaker_range_1$
         else
            call msg Calculating pitch range...
            call pitchrange_speakers
         endif
         call msg Calculating pitch range normalized pitch...
         call pitchrange_normalized_pitch
      endif
      if (syllables_available)	; hesitations used in duration data and boundary/prominence detection
         call msg Detecting hesitations... (hesitations provided by annotation = 'hesitation_available')
         call detect_hesitations anal_t1 anal_t2 annotation
      endif
      if (calc_prominence)
          call msg Calculating prominence...
          call calculate_prominence_measures anal_t1 anal_t2
      endif

      if (show_pseudosyllables)
        call grid_append_tier nucleiID syllable_tier newgridID
      endif
      if (contour_annotation)
         call msg Tonal annotation...
         call levels_analysis anal_t1 anal_t2
         call grid_append_tier nucleiID contour_tier newgridID
         if (not segfile_available)
            select newgridID
            Remove tier... 1	; dummy tier
         endif
         ; write contour TextGrid
         select nucleiID
         tmpID = Extract one tier... contour_tier
         Write to text file... 'contourfile$'
         call msg Tonal annotation saved to file... 'contourfile$'
         removeObject: tmpID
      endif
      if (boundary_annotation)
         show_prominence = 0		; plot prominence measures
         show_lengthening = 1		; plot lengthening
         boundary_annotation_verbose = 1
         save_intermediate_data = 1
         need_prosodic_profile = 0
         if (not (segm_type == segm_msyllvow or segm_type == segm_mrhyme or segm_type == segm_msyllpeak))
            call error_msg Boundary annotation requires Syllabic segmentation method 
            segm_type = segm_msyllvow
         endif
         if (corpus$ = "project_frfc")
            boundary_use_nuclei = 0	; 0= use pauses from annotation. 1= use gaps between nuclei 
         else
            boundary_use_nuclei = 1	; 0= use pauses from annotation. 1= use gaps between nuclei 
         endif
         call msg Calculating boundaries (boundary_use_nuclei='boundary_use_nuclei')...
         boundary_skip_hesit = 0 
         call boundary_analysis anal_t1 anal_t2 boundary_use_nuclei boundary_skip_hesit
         select nucleiID
         tmpID = Extract one tier... boundary_tier
         Write to text file... 'boundaryfile$'
         call grid_append_tier nucleiID boundary_tier newgridID
         call grid_append_tier nucleiID boundary_tier segmentationID
         call boundary_pass2 anal_t1 anal_t2
         call grid_append_tier nucleiID boundary2_tier newgridID
         call tier_get segmentationID "^boundary-manu$" ref_tier "Cannot find boundary reference tier in segmentation grid" 0
         call create_evaluation_data segmentationID ref_tier 'evalfile$' 
         call msg Boundaries ready
      endif

      if (stress_annotation)
         call stress_analysis anal_t1 anal_t2 pause_use_nuclei
         call grid_append_tier nucleiID stress_tier newgridID
      endif
      if (need_prosodic_profile)
         call prosodic_profile
      endif
      if (save_intermediate_data)
         if (not reuse_nucl)
            select nucleiID
            Write to text file... 'nuclfile$'
         endif
         if (not reuse_styl)
            select stylID
            Write to text file... 'stylfile$'
         endif
         call store_stylization stylID nucleiID 'targetsfile$'
         ; call store_stylization_resampled stylID nucleiID time_step 'stylpitchfile$'
         ;.outfname$ = prefix$ + "_dur.txt"		; uses basename
         ;.outfname$ = indir$ + "_dur_syllprom.txt"		; uses single output file
         ;call duration_data_from_textgrid segmentationID iFile "syllprom" "'.outfname$'" 'basename$'
      endif

      select stylID	; stylization pitch tier in Hz
      stylSTID = Copy... styl_ST
      call convert_Hz_ST stylSTID

    # Initialization for plotting results of current input file
      if (auto_pitchrange)
         ; GLOBAL automatic pitch range selection for entire corpus to be analysed
         ; call autorange stylSTID nucleiID anal_t1 anal_t2 1		; old code
         call speaker_autorange anal_t1 anal_t2
         ySTmax = ymax
         ySTmin = ymin
      endif
   else ; no stylization needed
       if (pitch_available)
          select pitchID
          y_ = Get mean... 0 0 semitones re 1 Hz
          ySTmin = y_ - 12
          ySTmax = y_ + 12
       endif
   endif ; need_stylization

# Draw all prosograms for current input file
   if (draw_prosograms)
      if (task == task_interactive)
         call gr_start_demowin 'basename$'
         call gr_run_demowin anal_t1 anal_t2 timeincr ySTmin ySTmax
         call cleanup_current_file
         call cleanup_global
         exit
      else
         call gr_start_picturewin
         if (nrof_pages == 0)
            # grid takes part of plotting area, so adjust lower Y-value
            ySTbottom = ySTmin - nrofplottedtiers*(ySTmax-ySTmin)/4
            call from_ST_rel_1_to_Hz ySTmin
            yHzmin = result
            call from_ST_rel_1_to_Hz ySTmax
            yHzmax = result
            ; call msg Drawing with Y-range 'ySTmin:1'-'ySTmax:1'ST ('yHzmin:0'-'yHzmax:0' Hz)
            call gr_first_viewport_of_page
            nrof_pages = 1
         endif
         call gr_write_all_prosograms anal_t1 anal_t2 timeincr
      endif
   endif ; draw_prosograms

   if (task == task_annotation)
      call gr_start_picturewin
      if (nrof_pages == 0)
         call gr_first_viewport_of_page
         nrof_pages = 1
         call gr_write_all_annotation anal_t1 anal_t2 timeincr
      endif
   endif

   ;if (save_intermediate_data)
      ; this step must follow plotting, because save_spreadsheet removes columns
         if (not collect_output)
            call store_features 1 0 'sheetfile$'
         else
            call store_features iFile 0 'outputfname$'
         endif
   ;endif

   label file_done
    # Delete temporary files for current input file
        call cleanup_current_file
   if (iFile = nrofFiles)
      # Write last output file if necessary
         if (task != task_interactive and draw_prosograms)
            if (nrof_plots > 1)
               call gr_write_prosogram
            endif
         endif
   endif
endproc


procedure map_segmentation_type .method$
   needs_segm_tg = 1		; selected segmentation type needs segmentation TextGrid
   needs_phon_tier = 0		; selected segmentation type needs phoneme tier
   needs_syll_tier = 0		; selected segmentation type needs syllable tier
   segmentation_name$ = .method$
   if (.method$ = "vow-nucl")
      segm_type = segm_vnucl
      needs_phon_tier = 1
   elsif (.method$ = "extern")
      segm_type = segm_extern
   elsif (.method$ = "loudness")		; automatic, loudness peaks
      segm_type = segm_aloudness
      needs_segm_tg = 0
   elsif (.method$ = "int-BP")	; automatic, peaks in bandpass filters speech
      segm_type = segm_anucl
      needs_intbp = 1
      needs_segm_tg = 0
   elsif (.method$ = "asyll")
      segm_type = segm_asyll
      needs_intbp = 1
      needs_segm_tg = 0
   elsif (.method$ = "rhyme")
      segm_type = segm_mrhyme
      needs_phon_tier = 1
      needs_syll_tier = 1
   elsif (.method$ = "syll+vow")
      segm_type = segm_msyllvow
      needs_phon_tier = 1
      needs_syll_tier = 1
   elsif (.method$ = "syll")
      segm_type = segm_msyllpeak
      needs_syll_tier = 1
   elsif (.method$ = "voiced")
      segm_type = segm_voiced
      needs_segm_tg = 0
   else
      segmentation_name$ = "unknown"
      call fatal_error Unknown segmentation type: '.method$'
   endif
   if (boundary_annotation)
         call map_segmentation_type '.segmentation_name$'
         if (not (segm_type == segm_msyllvow or segm_type == segm_mrhyme or segm_type == segm_msyllpeak))
            call error_msg Boundary annotation requires Syllabic segmentation method 
            segm_type = segm_msyllvow
         endif
   endif
endproc


procedure task_flags
# set some flags depending on task type
   need_parameters = 1
   need_stylization = 1
   needs_loudness = 0
   needs_intbp = 0
   needs_pitchrange = 0
   if (variableExists ("show_pitchrange"))
      if (show_pitchrange)
         needs_pitchrange = 1
      endif
   endif
   need_prosodic_profile = 0
   needs_segm_tg = 0
   needs_segm_type = 0
   draw_prosograms = 1
   show_vuv = 1
   show_intensity = 1
   show_intbp = 1
   show_lengthening = 0		; don't plot lengthening
   if (task == task_calc_pitch or task == task_calc_bp_intensity or task == task_calc_loudness or task == task_annotation)
      need_parameters = 0
      volatile = 0
      need_stylization = 0
      draw_prosograms = 0
   elsif (task == task_profile)
      needs_segm_type = 1
      need_prosodic_profile = 1
      draw_prosograms = 0
      needs_pitchrange = 1
   elsif (task == task_pitch_plot)
      need_stylization = 0
      segmentation_name$ = ""
      show_vuv = 0
      show_intensity = 0
      show_intbp = 0
      draw_prosograms = 1
      plot_pauses = 0
   elsif (task == task_interactive)
      needs_segm_type = 1
      draw_prosograms = 1
      rich = 0
      save_intermediate_data = 0
   elsif (task == task_segmentation)
      needs_segm_type = 1
      need_stylization = 0
      draw_prosograms = 0
      segm_type = segm_asyll
   elsif (task == task_validate_tiers)
      needs_segm_tg = 1
      need_parameters = 0
      need_stylization = 0
      draw_prosograms = 0
   elsif (task == task_autoanno)
      needs_segm_type = 1
      draw_prosograms = 1
      show_prominence = 0		; plot prominence measures
      if (boundary_annotation)
         calc_prominence = 1
         show_lengthening = 1		; plot lengthening
         boundary_annotation_verbose = 1
         save_intermediate_data = 1
         need_prosodic_profile = 0
      endif
      if (contour_annotation)
         needs_pitchrange = 1
         need_prosodic_profile = 0
      endif
      if (stress_annotation)
         calc_prominence = 1
         show_prominence = 1		; plot prominence measures 
         pause_use_nuclei = 1		; 1= use gaps between nuclei; 0= use pauses from annotation
      endif
   elsif (task == task_prosogram)
      needs_segm_type = 1
      if not (batch_mode)
         need_prosodic_profile = 1
      endif
      draw_prosograms = 1
   endif
endproc


procedure process_form
; Interpret all fields from script form and set variables accordingly
   needs_segm_type = 0		; true if task requires a segmentation type
   if (index (task$, "Recalculate pitch") > 0)
      task = task_calc_pitch
   elsif (index (task$, "Recalculate intensity") > 0)
      task = task_calc_bp_intensity
   elsif (index (task$, "Recalculate loudness") > 0)
      task = task_calc_loudness
   elsif (index (task$, "Prosogram") > 0)
      task = task_prosogram
      needs_segm_type = 1
   elsif (index (task$, "Prosodic profile only") > 0)
      task = task_profile
      needs_segm_type = 1
   elsif (index (task$, "Plot pitch") > 0)
      task = task_pitch_plot
   elsif (index (task$, "automatic segmentation") > 0)
      task = task_segmentation
      segm_type = segm_asyll
      segmentation_name$ = "asyll"
      needs_segm_type = 1
   elsif (index (task$, "Interactive") > 0)
      task = task_interactive
      needs_segm_type = 1
   elsif (index (task$, "Draw annotation"))
      task = task_annotation
   elsif (index (task$, "boundary"))
      task = task_autoanno
      boundary_annotation = 1
      needs_segm_type = 1
   elsif (index (task$, "contour"))
      task = task_autoanno
      contour_annotation = 1
      needs_segm_type = 1
   elsif (index (task$, "Validate"))
      task = task_validate_tiers
   else
      call fatal_error Invalid task
   endif

; Map segmentation type descriptions in form to names that will also be displayed in prosogram graph.
   if (needs_segm_type)
      .method$ = segmentation_method$
      if (index (.method$, "in vowels"))
         segmentation_name$ = "vow-nucl"
      elsif (index (.method$, "external"))
         segmentation_name$ = "extern"
      elsif (index (.method$, "loudness"))		; automatic, loudness peaks
         segmentation_name$ = "loudness"
      elsif (index (.method$, "BP-filtered"))		; automatic, peaks in bandpass filters speech
         segmentation_name$ = "int-BP"
      elsif (index (.method$, "Automatic: acoustic syllables"))
         segmentation_name$ = "asyll"
      elsif (index (.method$, "in rhyme"))
         segmentation_name$ = "rhyme"
      elsif (index_regex (.method$, "in syllables.*and vowels"))
         segmentation_name$ = "syll+vow"
      elsif (index_regex (.method$, "in syllables.*and local peak"))
         segmentation_name$ = "syll"
      elsif (index (.method$, "Automatic: voiced portions"))
         segmentation_name$ = "voiced"
      else
         call fatal_error Unknown segmentation type: '.method$'
      endif
   endif

; time_step (frame rate) used for calculation of intensity and pitch
   time_step = 'frame_period$'
   anal_t1 = left_Time_range
   anal_t2 = right_Time_range
; View
   s$ = left$ (view$, index(view$, ":") -1)
   view = 's$'
   rich = 0
   if (index (view$, "rich"))
      rich = 1
   endif
   draw_pitch_target_values = 0				; Draw pitch target values (in ST) in prosogram
   if (index (view$, "pitch targets"))
      draw_pitch_target_values = 1
   endif
   show_pitchrange = 0						; Show pitch range
   if (index (view$, "pitch range"))
      show_pitchrange = 1
   endif
; Thresholds
   j = index_regex (thresholds$, "/T")
   s$ = mid$ (thresholds$, 3, j-3)		; s$ = glissando threshold(s)
   if (index (thresholds$, "adaptive"))		; adaptive glissando threshold (lower before pause)
      adaptive_glissando = 1
      s2$ = left$ (s$, 4)
      glissando_low = 's2$'
      s2$ = mid$ (s$, 6, 4)
      glissando = 's2$'
   else						; fixed glissando threshold
      adaptive_glissando = 0
      glissando = 's$'
      glissando_low = glissando
   endif
   diffgt = extractNumber (thresholds$, "DG=")
   mindur_ts = 0.035			; Minimum duration for a tonal segment (default)
   j = index (thresholds$, "dmin")
   if (j > 0)
      s$ = mid$ (thresholds$, j+5, length (thresholds$) -(j+4))
      mindur_ts = 's$'
   endif
   if (not variableExists ("output_mode"))
      output_mode$ = "Fill page with strips"
   endif
   outputmode$ = left$ (output_mode$, index(output_mode$, " ") -1)
   volatile = 0				; by default, process full signal and store results
   if (index (parameter_calculation$, "Partial") > 0)
      volatile = 1
   endif
   scale_signal_amplitude = 0
   if (variableExists ("scale_signal_intensity"))	; Form contains field "Scale signal intensity"
      if (index (scale_signal_intensity$, "Scale"))
         scale_signal_amplitude = 1
      endif
   endif
   clip_level = 0
   if (variableExists ("clipping_threshold"))		; Form contains field "Clipping threshold"
      if (clipping_threshold > 100 or clipping_threshold < 0)
         call fatal_error "Invalid value for clipping threshold"
      endif
      clip_level = clipping_threshold
   endif

   minimum_pitch = left_F0_detection_range
   maximum_pitch = right_F0_detection_range
   if (left_Pitch_range == 0)
      auto_pitchrange = 1		; automatic pitch range adjustment
   else
      auto_pitchrange = 0		; manual pitch range adjustment
      if (right_Pitch_range <= left_Pitch_range or left_Pitch_range <= 0)
         call fatal_error "Invalid values for pitch range"
      endif
   endif
   ySTmin = left_Pitch_range
   ySTmax = right_Pitch_range

   if (minimum_pitch < 40 or maximum_pitch > 800)
      call fatal_error "Invalid F0 range: expected to be within 40 - 800 Hz range"
   endif
   if (minimum_pitch >= maximum_pitch)
      call fatal_error "Invalid F0 range: lower limit > higher limit"
   endif
   timeincr = time_interval_per_strip	; time increment for plot

   call task_flags

   if (draw_prosograms and (index (output_format$,"EMF")) and not windows)
      call error_msg Windows Metafiles (EMF) are supported on Windows systems only
   endif
   if (draw_prosograms and (index (output_format$,"JPG")))
      if (not fileReadable (path_ghostscript$))
         call error_msg JPG output requires Ghostscript, which is not found. Verify configuration for Ghostcript.
      endif
   endif

   s$ = input_files$
   s$ = replace$ (s$, " ", "", 0)
   if (length(s$) < 1)
      input_files$ = chooseReadFile$ ("Select file")
   endif
endproc


procedure initialization_multiple_files
   collect_output = 0	; collect output of duration calculation in a single output file

   # Unit (ST or Hz) used for expressing shown pitch range
      units$ = "Semitones"
   ySTmin = 0
   ySTmax = 100
   # diffST = difference in ST between ST-scale relative to 100Hz and that rel to 1Hz
      diffST = 12 * log2(100/1)
   if (units$ = "Semitones")
      yHzmin = semitonesToHertz(ySTmin-diffST)
      yHzmax = semitonesToHertz(ySTmax-diffST)
   else
      ySTmin = hertzToSemitones(yHzmin) - hertzToSemitones(1)
      ySTmax = hertzToSemitones(yHzmax) - hertzToSemitones(1)
   endif

; for drawing: start counters for files and pages
   call gr_init
   file_stamp = 1	; Show input speech filename on prosogram
   plot_version = 1	; Show version number
   file_ctr = 1	; counter for EPS/EMF filename
   nrof_pages = 0

; for globalsheet
   if (need_prosodic_profile)
      call create_table_global_report
   endif
endproc


procedure initialization_per_file
   needs_loudness = 0		; 1 when loudness is needed for prominence measurement
   signal_available = 0		; speech signal loaded
   intensity_available = 0
   pitch_available = 0
   harmonicity_available = 0
   intbp_available = 0
   inthp_available = 0
   nucldat_available = 0
   loudness_available = 0
   segfile_available = 0	; annotation file with alignements of phonemes, syllables, etc.
   segmentation_available = 0	; 1 if segmentation has been read from saved <basename>_nucl.TextGrid file
   stylization_available = 0	; 1 if stylization has been read from saved <basename>_styl.PitchTier file
   reuse_nucl = 0			; 1 if nucleus file could be read and used as segmentation
   reuse_styl = 0
   phones_available = 0		; 1 if phon tier is found
   syllables_available = 0	; 1 if syll tier is found
   creak_available = 0		; creak textgrid/tier found and read ?
   speaker_available = 0	; speaker textgrid/tier found and read ?
   hesitation_available = 0	; hesitation textgrid/tier found and read ?
   newgrid_available = 0
endproc


procedure construct_filenames fname$
   if (rindex (fname$,".") == 0)
      call fatal_error Invalid filename for input file. Should include filename extension.
   endif
   call fname_parts 'fname$'
   fext$ = replace_regex$ (result3$, "(.)", "\L\1", 0)	; to lowercase
   if (task <> task_validate_tiers)
      if (index(":wav:aiff:aifc:nist:flac:sound:", ":'fext$':") == 0)
         printline FileExtension=<'fext$'>
         call fatal_error Input file should be an audio file supported by Praat Open LongSound, with extension .wav, .aiff, .aifc, .nist, or .flac
      endif
   endif
   basename$    = result2$
   indir$       = result4$
   prefix$      = indir$ + basename$
   signalfile$  = prefix$ + "." + fext$
   pitchfile$   = prefix$ + ".Pitch"
   intensityfile$ = prefix$ + ".Intensity"
   harmonicityfile$ = prefix$ + ".Harmonicity"
   segfile$		= prefix$ + ".TextGrid"			; TextGrid with already available segmentation
   autosegfile$	= prefix$ + "_auto.TextGrid"		; automatic segmentation output
   nuclfile$	= prefix$ + "_nucl.TextGrid"		; nuclfile contains automatic segmentation etc.
   ;nucldatfile$= prefix$ + "_ndat.TableOfReal"		; nucldat contains features per syllable
   creakfile$	= prefix$ + "_creak.TextGrid"
   speakerfile$	= prefix$ + "_speaker.TextGrid"
   contourfile$	= prefix$ + "_contour.TextGrid"
   boundaryfile$ = prefix$ + "_boundary.TextGrid"
   intbpfile$	= prefix$ + "_BP.Intensity"
   inthpfile$	= prefix$ + "_HP.Intensity"
   loudnessfile$ = prefix$ + "_loud.Intensity"
   # The following files are created in the data directory and deleted on exit unless "Save intermediate data"
   stylfile$	= prefix$ + "_styl.PitchTier"
   stylpitchfile$ = prefix$ + "_styl.Pitch"
   targetsfile$	= prefix$ + "_styl.txt"
   statsfile$	= prefix$ + "_profile.txt"
   sheetfile$	= prefix$ + "_data.txt"
   globalfile$	= indir$  + "globalsheet.txt"
   evalfile$	= prefix$ + "_eval.txt"

   if (variableExists ("output_filename$"))
      output_fname$ = output_filename$	; local copy of specification given on script form
      if (length (indir$))
         output_fname$ = replace$ (output_fname$, "<input_directory>", indir$, 1)
      else
         output_fname$ = replace$ (output_fname$, "<input_directory>/", "", 1)
      endif
      output_fname$ = replace$ (output_fname$, "<basename>", basename$, 1)
      output_fname$ = replace$ (output_fname$, "//", "/", 1)
      if (single_fname_graphics_output)
        output_filename$ = output_fname$	; keep name for all graphics output files
      else
         file_ctr = 1	; counter for graphics filename
         nrof_pages = 0
      endif
   endif

   if (variableExists ("corpus$"))
      if (variableExists ("prosoview"))
         call prosoview_construct_filenames
      elsif (length (corpus$))
         if (variableExists ("corpus_subdir_pitch$"))
            pitchfile$	= corpus_home$ + corpus_subdir_pitch$ + basename$ + ".Pitch"
         endif
         if (variableExists ("corpus_subdir_tg$"))
            segfile$	= corpus_home$ + corpus_subdir_tg$ + basename$ + ".TextGrid"
         endif
         if (variableExists ("corpus_subdir_data$"))
            stylfile$		= corpus_home$ + corpus_subdir_data$ + basename$ + "_styl.PitchTier"
            stylpitchfile$	= corpus_home$ + corpus_subdir_data$ + basename$ + "_styl.Pitch"
            targetsfile$	= corpus_home$ + corpus_subdir_data$ + basename$ + "_styl.txt"
            nuclfile$		= corpus_home$ + corpus_subdir_data$ + basename$ + "_nucl.TextGrid"
            statsfile$		= corpus_home$ + corpus_subdir_data$ + basename$ + "_profile.txt"
            sheetfile$		= corpus_home$ + corpus_subdir_data$ + basename$ + "_data.txt"
            contourfile$	= corpus_home$ + corpus_subdir_data$ + basename$ + "_contour.TextGrid"
            globalfile$		= corpus_home$ + corpus_subdir_data$ + "globalsheet.txt"
         endif
         if (variableExists ("corpus_subdir_img$"))
            output_fname$ = corpus_home$ + corpus_subdir_img$ + basename$ + "_"
         endif
      endif
   endif
endproc


procedure read_annotation_file
   segfile_available = 0
   if (fileReadable (segfile$))
      segfile_available = 1
   else
      call msg read_annotation_file: Cannot open TextGrid file with segmentation: <'segfile$'>
   endif
   if ((not segfile_available) and needs_segm_tg)
      call fatal_error Cannot open TextGrid with segmentation from file <'segfile$'>
   endif
   if (segfile_available)
      call msg Reading annotation file "'segfile$'"
      segmentationID = Read from file... 'segfile$'
      Rename... annotation_input
   endif
endproc


procedure corpus_conversion .step .gridname$ .corpus$ contourfile$
# In-place conversion of a TextGrid object
# <.step>	1 = before preparation of plotted tiers, 2 = after preparation
   .grid = '.gridname$'
   select .grid

   if (.corpus$ = "cprom" and .step = 1)
      @process_cprom (.grid)
   endif

   if (.corpus$ = "gvlex" and .step = 1)
      @lpa_sampa (.grid, corpus_tier_phon$)
      @lpa_sampa (.grid, corpus_tier_syll$)
   endif

   if (.corpus$ = "RUHRCAT" and .step = 2)
      call tier_number_by_name .grid "phon"
      if (result)
         Replace interval text... result 0 0 "^j([aOoueE@])" "i\1" Regular Expressions
         Replace interval text... result 0 0 "^w([o0iaeE])" "u\1" Regular Expressions
         Replace interval text... result 0 0 "^w\\ct" "uO" Regular Expressions
         Replace interval text... result 0 0 "^j\\ct" "iO" Regular Expressions
         Replace interval text... result 0 0 "^j\\ef" "iE" Regular Expressions
         Replace interval text... result 0 0 "(\\:f)*" "" Regular Expressions
         Replace interval text... result 0 0 "^mmm$" "m=" Regular Expressions
      endif
      call tier_number_by_name .grid "syll"
      .syll_tier = result
      if (result)
         Replace interval text... result 0 0 "^mmm$" "m=" Regular Expressions
      endif
  ; Create prominence and hesitation tiers from "DM+Hes" tier
      call tier_number_by_name .grid "DM.Hes"
      if (result)
         .deliv_tier = result
         .nt = Get number of tiers
         .hes_tier = .nt + 1
         Duplicate tier... .syll_tier .hes_tier hes
         .ni = Get number of intervals... .syll_tier
         for .j to .ni
            .t1s = Get start point... .syll_tier .j
            .t2s = Get end point... .syll_tier .j
            call interval_from_time .grid .deliv_tier .t1s+(.t2s-.t1s)/2 interv
            .label$ = Get label of interval... .deliv_tier interv
            if (index (.label$, "Hes"))
              .s$ = "H"
            else
              .s$ = ""
            endif
            Set interval text... .hes_tier .j '.s$'
         endfor
      endif
   endif ; (.corpus$ = "RUHRCAT" and .step = 2)
endproc


procedure prepare_plotted_textgrid ltiers$
# Prepare textgrid for plot using tiers  
   nrofplottedtiers = 0
   if (segfile_available == 0 and contour_annotation)
      newgridID = Create TextGrid... signal_start signal_finish dummy
      newgrid_available = 1
   endif
   if (segfile_available)
      tiers = 0
      nrofTiers = Get number of tiers
      call tier_number_by_name segmentationID "^[Ss]peaker$"
      speaker_tier_in = result
      repeat
         call next_field 'ltiers$'
         if (result)			; next field found
            field$ = result2$
            if (left$ (field$, 1) = "*")
               field$ = right$ (field$, length(field$) - 1)
               convert = 1
            else
               convert = 0
            endif
            call is_number 'field$'
            if (result == 0)		; next field is not a number
               call tier_number_by_name segmentationID field$
               tier_in = result
               if (result == 0)		; tier name not found
                  call error_msg No tier named "'field$'" found in input textgrid. Change content of field "Tiers to show".
                  call error_msg This tier will be skipped in prosogram.
               endif
            else			; field is a number
               tier_in = 'field$'
               if (tier_in > nrofTiers)
                  call error_msg Tier 'tier_in' not found. Input textgrid has only 'nrofTiers' tiers. Change content of field "Tiers to show".
                  call error_msg This tier will be skipped in prosogram.
                  tier_in = 0
               endif
            endif
            if (tier_in > 0)		; valid tier to add
               select segmentationID
               if (tiers == 0)		; this is first tier to add to plotted grid
                  tmpID = Extract tier... tier_in
                  newgridID = Into TextGrid
                  Rename... plotted_tiers
                  newgrid_available = 1
                  removeObject (tmpID)
               else
                  call grid_append_tier segmentationID tier_in newgridID
                  select newgridID
                  Rename... plotted_tiers
               endif
               tiers += 1
               if (convert)
                  call convert_sampa_ipa newgridID tiers
               endif
            endif
         endif
         ltiers$ = result3$
      until (result == 0)
      nrofplottedtiers = tiers
   endif ; segfile_available
   if (contour_annotation or show_pseudosyllables)
      nrofplottedtiers += 1		; results will appear in extra tier
   endif
   if (boundary_annotation)
      nrofplottedtiers += 1		; results will appear in extra tier
   endif
   if (stress_annotation)
      nrofplottedtiers += 1		; results will appear in extra tier
   endif
endproc


procedure peek_signal
   call fname_parts 'signalfile$'
   .fext$ = replace_regex$ (result3$, "(.)", "\L\1", 0)	; to lowercase
   if (.fext$ = "sound")
      Read from file: signalfile$
   else
      Open long sound file: signalfile$
   endif
   signal_start = Get starting time
   signal_finish = Get finishing time
   Remove
endproc


procedure read_signal
# Read signal. Apply clipping if clip > 0. Apply scaling of (scale_signal_amplitude > 0).
   soundID = Read from file... 'signalfile$'
   fullsoundID = soundID
   signal_start = Get starting time
   signal_finish = Get finishing time
   if (clip_level > 0)	; Avoid error related to speech files clipped at max amplitude.
      call msg Clipping speech signal at 'clip_level' percent...
      clip = clip_level/100
      Formula... if (abs(self)>clip) then if self>0 then clip else -clip fi else self fi
   endif
   if (scale_signal_amplitude)
      call msg Scaling speech signal at 70 dB ...
      Scale intensity... 70.0
   endif
   signal_available = 1
endproc


procedure read_parameter_files
   if (fileReadable (pitchfile$))
      pitchID = Read from file: pitchfile$
      .t = Get time step
      if (.t > time_step)
         call msg Pitch file on disk has larger time step.
         Remove
      else
         .t2 = Get end time
         if (.t2 + 5*time_step < signal_finish)
            call msg Pitch file end time ('.t2' s) < speech signal end time ('signal_finish' s)
         endif
         pitch_available = 1
         call msg Read pitch from file "'pitchfile$'" (time step='.t:3')
      endif
   endif
   if (show_harmonicity)
      if (fileReadable (harmonicityfile$))
         harmonicityID = Read from file... 'harmonicityfile$'
         harmonicity_available = 1
      endif
   endif
   if (needs_loudness or include_loudness)
      if (fileReadable (loudnessfile$))
         loudnessID = Read from file... 'loudnessfile$'
         loudness_available = 1
      endif
   endif
   if (needs_intbp)
      if (fileReadable (intbpfile$))
         intbpID = Read from file... 'intbpfile$'
         .t = Get time step
         intbp_available = 1
         call msg Read BP intensity from file "'intbpfile$'" (time step='.t:3')
      endif
   endif
endproc


procedure calculate_pitch_int
   if (not signal_available)
      call read_signal
   endif
; Standard settings, see FAQ Pitch Analysis
   tstep = 0			; automatic, i.e. 0.75/pitch_floor, e.g. for 60Hz, time step = 0.0125 s
   voicing_threshold = 0.45
   silence_threshold = 0.03
   octave_cost = 0.01
   octave_jump_cost = 0.35
   vuv_cost = 0.14
; Modified settings
   tstep = time_step		; typically 0.01 or 0.005 s
   ;voicing_threshold = 0.2	; setting for v2.8
   ;voicing_threshold = 0.35	; setting for v2.7
   ;octave_jump_cost = 0.5

   call from_Hz_to_ST_rel_1 minimum_pitch
   .minST = result
   call from_Hz_to_ST_rel_1 maximum_pitch 
   .maxST = result
   call msg Calculating pitch... 'newline$'Time step='tstep:2', F0 range='minimum_pitch'-'maximum_pitch' Hz ('.minST:1'-'.maxST:1' ST), Voicing threshold='voicing_threshold:2'
   select soundID
   pitchID = To Pitch (ac)...  tstep minimum_pitch 15 no silence_threshold voicing_threshold octave_cost octave_jump_cost vuv_cost maximum_pitch
   .p50 = Get quantile... 0 0 0.50 Hertz
   .p95 = Get quantile... 0 0 0.95 Hertz
   .p05 = Get quantile... 0 0 0.05 Hertz
   call msg Pitch percentiles 50%='.p50:0' 5%='.p05:0' 95%='.p95:0' (Hz)
   if (not volatile)
      call msg Writing Pitch object to 'pitchfile$'
      Write to binary file... 'pitchfile$'
   endif
   pitch_available = 1
endproc


procedure calculate_intensity_bp_filtered
   call msg Calculating intensity of BP filtered signal...
   fc_low = 300
   fc_high = 3500
;printline avoid_insuff_memory='avoid_insuff_memory'
   if (avoid_insuff_memory)
      call intensity_bp_filter_longsound_compact "'signalfile$'" fc_low fc_high intbpID
   else
      if (not signal_available)
         call read_signal
      endif
      select soundID
      tmpfsID = Filter (pass Hann band)... fc_low fc_high 100
      select tmpfsID
      intbpID = To Intensity... 100 time_step
      intbpID = selected ("Intensity", -1)		; keep this line !!
      removeObject (tmpfsID)
   endif
   if (not volatile)
      select intbpID
      Write to text file... 'intbpfile$'
   endif
   intbp_available = 1
endproc


procedure bp_filter .soundin .fc_low .fc_high .varname$
# BP-filter sound object and put reference to resulting object in variable with name <.varname>
   select .soundin
   .out = Filter (pass Hann band)... .fc_low .fc_high 100
   '.varname$' = .out
endproc


procedure intensity_bp_filter_longsound_compact .fname$ .fc_low .fc_high .varname$
# Filter signal and compute intensity. Filtered signal is temporarily stored as file. 
# Use this procedure to avoid insufficient memory.
# Use double quotes if filename contains blanks:
#   call intensity_bp_filter_longsound_compact "'filename$'" 300 2500 myvar
# .fname$	filename of sound file to be filtered
   .tmpfile$ =  prefix$ + "_filtered.wav"
   .longsound = Open long sound file... '.fname$'
   .t1 = Get start time
   .endtime = Get end time
   .maxdurpart = 300		; duration of part of signal to be filtered separatedly due to available memory
   .partnr = 1
   repeat
      .t2 = min (.endtime, .t1 + .maxdurpart)
      select .longsound
      .part = Extract part... .t1 .t2 yes
      if (clip_level > 0)
         clip = clip_level/100
         Formula... if (abs(self)>clip) then if self>0 then clip else -clip fi else self fi
      endif
      .filt = Filter (pass Hann band)... .fc_low .fc_high 100
      Formula... if (abs(self)>0.98) then if self>0 then 0.98 else -0.98 fi else self fi
      if (.partnr == 1)
         Save as WAV file... '.tmpfile$'
      else
         Append to existing sound file... '.tmpfile$'
      endif
      select .filt
      plus .part
      Remove
      .t1 = .t2
      .partnr += 1 
   until (.t2 >= .endtime)
   select .longsound				; free memory
   Remove
   .tmp = Read from file... '.tmpfile$'
   intbpID = To Intensity... 100 time_step
   intbpID = selected ("Intensity", -1)		; keep this line !!
   '.varname$' = intbpID
   select .tmp
   Remove
   filedelete '.tmpfile$'
endproc


procedure calculate_loudness
   if (not signal_available)
      call read_signal
   endif
   execute loudness.praat 'soundID' 'signal_start' 'signal_finish' 'time_step'
   loudnessID = selected ("Intensity", -1)
   if (not volatile)
      Write to binary file... 'loudnessfile$'
   endif
   loudness_available = 1
endproc


procedure calculate_parameters
   fc_low = 300
   fc_high = 3500
   if (not signal_available)
      call read_signal
   endif
   if (volatile)
   # Calculate parameters on the fly only for interval to stylize
      select soundID
      tmpsoundID = Extract part... anal_t1 anal_t2 Rectangular 1.0 yes
      removeObject (soundID)	; remove full signal
      soundID = tmpsoundID	; redefine soundID
      select soundID
      if (scale_signal_amplitude)
         Scale intensity... scale_signal_amplitude
      endif
      intensityID = To Intensity... 100 time_step
      intensityID = selected ("Intensity", -1)		; keep this line !!
      intensity_available = 1
      call calculate_pitch_int
      if (needs_loudness and not loudness_available)
         execute loudness.praat 'soundID' 'anal_t1' 'anal_t2' 'time_step'
         loudnessID = selected ("Intensity", -1)
         loudness_available = 1
      endif
      if (needs_intbp and not intbp_available)
         select soundID
         tmpfsID = Filter (pass Hann band)... fc_low fc_high 100
         select tmpfsID
         intbpID = To Intensity... 100 time_step
         intbpID = selected ("Intensity", -1)		; keep this line !!
         removeObject (tmpfsID)
         intbp_available = 1
      endif
   else ; (volatile == 0)
      if (not intensity_available)

          if (corpus$ = "rhapsodie")
             prefilterHP100 = 0
             if (basename$ = "Rhap-D0008")
                prefilterHP100 = 1
             endif
          endif

    if (prefilterHP100)	; Apply HP filtering to avoid problems with low quality recordings by filtering low frequency band
        printline Preprocessing (high-pass filtering) speech signal... 'basename$'
        select soundID
        fc_low = 100
        tmpID = Filter (pass Hann band)... fc_low 0 100
        removeObject (soundID)
        soundID = tmpID
    endif
    if (prefilterLP)	; Apply LP filtering when fricatives are very intense
        printline Preprocessing (low-pass filtering) speech signal... 'basename$'
        select soundID
        fc = 2500
        tmpID = Filter (pass Hann band)... 0 fc 100
        removeObject (soundID)
        soundID = tmpID
    endif

        select soundID
        tmpID = To Intensity... 100 time_step
        intensityID = selected ("Intensity", -1)		; keep this line !!
        intensity_available = 1
        ; max_intensity = Get maximum... 0 0 None
      endif
      if (not pitch_available)
        call calculate_pitch_int
      endif
      if (show_harmonicity and not harmonicity_available)
        select soundID
        silence_threshold = 0.1
        nrofperiods = 1.0
        harmonicityID = To Harmonicity (cc)... time_step minimum_pitch silence_threshold nrofperiods
        harmonicity_available = 1
      endif
      if (needs_loudness and not loudness_available)
        execute loudness.praat 'soundID' 'signal_start' 'signal_finish' 'time_step'
        loudnessID = selected ("Intensity", -1)
        Write to short text file... 'loudnessfile$'
        loudness_available = 1
      endif
      if (needs_intbp and not intbp_available)
        call calculate_intensity_bp_filtered
      endif
    endif ; # volatile
    # Calculate V/UV decision and store in TextGrid
      select pitchID
      pointprocID = To PointProcess
      vuvgridID = To TextGrid (vuv)... 0.02 0.01
      Rename... vuv
      removeObject (pointprocID)
endproc


procedure peek_settings
# Read settings from intermediate data file (nuclfile).
   if (not fileReadable (nuclfile$))
      call fatal_error Cannot find nucleus file: 'nuclfile$' 'newline$'Calculate it first using Prosogram with option "Save intermediate data".
   else
      ; call msg Found nucleus file 'nuclfile$'
      tmpID = Read from file: nuclfile$
      call tier_number_by_name tmpID "^settings$"
      if (result)
         .s$ = Get label of interval... result 1
         segmentation_name$ = extractWord$ (.s$, "SEG=")
         adaptive_glissando = extractNumber (.s$, "ADAPT=")
         glissando = extractNumber (.s$, "GT=")
         glissando_low = glissando
         if (adaptive_glissando)
            glissando_low = extractNumber (.s$, "GT_low=")
         endif
         diffgt = extractNumber (.s$, "DG=")
         mindur_ts = extractNumber (.s$, "MINTS=")
      else
         call fatal_error Cannot find segmentation type in nucleus file.
      endif
      removeObject: tmpID
   endif
endproc


procedure segmentation method
# Calculate segmentation or read it from file 
   if (method == segm_vnucl)		; vowel nuclei
      segmentation_name$ = "vow-nucl"
   elsif (method == segm_extern)
      segmentation_name$ = "extern"
   elsif (method == segm_aloudness)	; automatic, loudness peaks
      segmentation_name$ = "loudness"
   elsif (method == segm_anucl)		; automatic, peaks in bandpass filters speech
      segmentation_name$ = "int-BP"
   elsif (method == segm_msyllvow)	; syllabic nuclei
      segmentation_name$ = "syll+vow"
   elsif (method == segm_msyllpeak)	; syllabic nuclei
      segmentation_name$ = "syll"
   elsif (method == segm_mrhyme)	; syllable rhyme
      segmentation_name$ = "rhyme"
   elsif (method == segm_asyll)		; pseudo-syllables
      segmentation_name$ = "asyll"
   elsif (method == segm_voiced)	; voiced portions
      segmentation_name$ = "voiced"
   else
      segmentation_name$ = "unknown"
   endif

   if (task == task_interactive)
      if (not fileReadable (nuclfile$))
         call msg Interactive mode: cannot find nucleus file 'nuclfile$'
      else
         call msg Interactive mode: found nucleus file 'nuclfile$'
         tmpID = Read from file: nuclfile$
         call tier_number_by_name tmpID "^settings$"
         if (result)
;printline interactive mode: found settings
            settings_tier = result
            s$ = Get label of interval... settings_tier 1
            method_$ = extractWord$ (s$, "SEG=")
            t1_ = extractNumber (s$, "t1=")
            t2_ = extractNumber (s$, "t2=")
            gt_ = extractNumber (s$, "GT=")
            gt_low_ = extractNumber (s$, "GT_low=")
            if (gt_low_ == undefined)
              gt_low_ = gt_
            endif
            .adapt = extractNumber (s$, "ADAPT=")
            if (.adapt == undefined)
              .adapt = 0
            endif
            dg_ = extractNumber (s$, "DG=")
            mints_ = extractNumber (s$, "MINTS=")
;printline interactive mode: anal_t1='anal_t1' t1='t1_'
;printline interactive mode: anal_t2='anal_t2' t2='t2_'
            if (anal_t1 >= t1_ and abs(anal_t2 - t2_) < time_step) ; Correct for rounding errors in 32 vs. 64 bit 
;printline interactive mode: times match: anal_t1='anal_t1:3' anal_t2='anal_t2:3'
              ;;if (method_$ = segmentation_name$)
              segmentation_name$ = method_$
;printline interactive mode: segmentation types match: method_='method_$' segmentation_name='segmentation_name$'
                  call tier_number_by_name tmpID "^phone$"
                  phone_tier = result
               if (result)
                  call tier_number_by_name tmpID "^syll$"
                  syllable_tier = result
               endif
               if (result)
                  call tier_number_by_name tmpID "^nucleus$"
                  nucleus_tier = result
               endif
               if (result)
                  call tier_number_by_name tmpID "^vuv$"
                  vuv_tier = result
                  removeObject (vuvgridID)
               endif
               if (result)
                  call tier_number_by_name tmpID "^discont$"
                  discontinuity_tier = result
               endif
               if (result)
                  call tier_number_by_name tmpID "^safe$"
                  safe_tier = result
               endif
               if (result)
                  call tier_number_by_name tmpID "^pointer$"
                  pointer_tier = result
               endif
               if (result)
                  call tier_number_by_name tmpID "^speaker$"
                  speaker_tier = result
                  if (speaker_tier)
                     speaker_available = 1	; speaker tier available
                  endif
               endif
;printline interactive mode: result='result'
               if (result)		; success
                  reuse_nucl = 1
                  segmentation_available = 1
;printline interactive mode: success segmentation_available='segmentation_available' reuse_nucl='reuse_nucl'
                  nucleiID = tmpID
                  nrof_nuclei_analysed = Count labels... safe_tier a
                  ;if (task == task_interactive)
                  @gr_printline: "Loaded segmentation from file 'nuclfile$'"
                  call msg Loaded segmentation from file 'nuclfile$'
               endif ; if result
             ;;endif ; if method ...
           endif ; if anal_t1 ...
         endif ; tier with settings found
         if (not segmentation_available) ; textgrid read but not used
            removeObject: tmpID
         endif
      endif ; file readable
;printline interactive mode: segmentation_available='segmentation_available' reuse_nucl='reuse_nucl' nr='nrof_nuclei_analysed'
   endif ; task_interactive

   if not segmentation_available
       @gr_printline: "Copying annotation tiers..."
# Make segmentation into nuclei depending upon segmentation method
    # tiers in annotation TextGrid file
        phone_tier_in = 1	; default location
    # create TextGrid nucleiID
        phone_tier = 1
        dip_tier = 2
        nucleus_tier = 3
        syllable_tier = 4
        vuv_tier = 5
        discontinuity_tier = 6
        safe_tier = 7
        pointer_tier = 8
        speaker_tier = 9
        creak_tier = 10
        settings_tier = 11
        hesitation_tier = 12
        contour_tier = 0	; may be added later for contour_annotation
        boundary_tier = 0	; may be added later for automatic boundary detection
        stress_tier = 0		; may be added later for stress_annotation
        n = 12
        tiers$ = "phone dip nucleus syll vuv discont safe pointer speaker creak settings hesitation"
        point_tiers$ = "dip discont"
        if (boundary_annotation)
           tiers$ = tiers$ + " boundary-auto"
           point_tiers$ = point_tiers$ + " boundary-auto"
           n += 1
           boundary_tier = n
        endif
        if (contour_annotation)
           tiers$ = tiers$ + " contour"
           n += 1
           contour_tier = n
        endif
        if (stress_annotation)
           tiers$ = tiers$ + " stress"
           n += 1
           stress_tier = n
        endif
        nucleiID = Create TextGrid... signal_start signal_finish "'tiers$'" 'point_tiers$'
        Rename... nucl
        nt = Get number of tiers
    # copy VUV decision from vuvgrid to nuclei grid
        call copy_tier vuvgridID 1 nucleiID vuv_tier
        removeObject (vuvgridID)


   if (segfile_available) ; and not internal data format available
      call tier_number_by_name segmentationID "^phon"
      if (variableExists ("corpus$") and variableExists ("corpus_tier_phon$"))
         if (length (corpus_tier_phon$))
            call tier_number_by_name segmentationID "^'corpus_tier_phon$'$"
         endif
      endif
      if (result)
         phone_tier_in = result
         call msg Using phonetic alignment from tier named "'result2$'"
         phones_available = 1
      else
         phone_tier_in = 1	; assume tier 1 contains phonetic alignment
      endif
      call tier_replace segmentationID phone_tier_in nucleiID phone_tier nucleiID
      select nucleiID
      Set tier name... phone_tier phone
      call tier_number_by_name segmentationID "^syll"
      if (variableExists ("corpus$") and variableExists ("corpus_tier_syll$"))
         if (length (corpus_tier_syll$))
            call tier_number_by_name segmentationID "^'corpus_tier_syll$'$"
         endif
      endif
      if (result and segm_type <> segm_asyll)
      ; In segm_asyll, the syllable tier is used for pseudo-syllables and should not be used for true syllables.
         syll_tier_in = result
         call msg Using syllable alignment from tier named "'result2$'"
         call copy_tier segmentationID syll_tier_in nucleiID syllable_tier
;         call tier_replace segmentationID syll_tier_in nucleiID syllable_tier nucleiID
;         select nucleiID
;         Set tier name... syllable_tier syll
         syllables_available = 1
      endif
      call tier_number_by_name segmentationID "^[Ss]peaker$"
      if (variableExists ("corpus$") and variableExists ("corpus_tier_speaker$"))
         if (length (corpus_tier_speaker$))
            call tier_number_by_name segmentationID "^'corpus_tier_speaker$'$"
         endif
      endif
      if (result)
         speaker_tier_in = result
         call msg Using speaker information from tier named "'result2$'"
         if (corpus$ = "rhapsodie") 
            call copy_tier_tweeked segmentationID speaker_tier_in nucleiID speaker_tier
         else
            call copy_tier segmentationID speaker_tier_in nucleiID speaker_tier
         endif
         speaker_available = 1	; speaker tier available
      else			; input segmentation textgrid does not contain speaker tier 
         select nucleiID
         Set interval text... speaker_tier 1 ANON
      endif
      call tier_number_by_name segmentationID "^[Cc]reaky?$"
      if (result)
         call msg Using creak information from tier named "'result2$'" of "'segfile$'"
         call copy_tier segmentationID result nucleiID creak_tier
         creak_available = 1
      endif
      word_tier_in = 0
      call tier_number_by_name segmentationID "^word$"
      if (result)
         word_tier_in = result
      endif
      call tier_number_by_name segmentationID "^hes$"
      if (result)
         hesitation_tier_in = result
         call msg Using hesitation information from tier named "'result2$'" of "'segfile$'"
         call copy_tier segmentationID hesitation_tier_in nucleiID hesitation_tier
         hesitation_available = 1
      else
         call copy_tier nucleiID syllable_tier nucleiID hesitation_tier
         call tier_clear_text nucleiID hesitation_tier
      endif
   endif
   if (not speaker_available)
      if (fileReadable (speakerfile$))
         call msg Reading speaker information from 'speakerfile$' 
         tmpID = Read from file... 'speakerfile$'
         call tier_number_by_name tmpID "^[Ss]peaker$"
         if (result > 0)
            call copy_tier tmpID result nucleiID speaker_tier
            speaker_available = 1
         else
            call msg No speaker tier found in 'speakerfile$' 
         endif
         removeObject (tmpID)
      endif
   endif
   if (not creak_available)
      if (fileReadable (creakfile$))
         call msg Reading 'creakfile$' 
         tmpID = Read from file... 'creakfile$'
         call tier_number_by_name tmpID "^[Cc]reaky?$"
         if (result > 0)
            call msg Reading creak information from tier 'result' of "'creakfile$'"
            call copy_tier tmpID result nucleiID creak_tier
            creak_available = 1
         endif
         removeObject (tmpID)
      endif
   endif
   if (corpus$ = "cprom" or corpus$ = "rhapsodie")
         call tier_number_by_name segmentationID "^contour$"
         if (result)
            call grid_append_tier segmentationID result nucleiID
         endif
         call tier_number_by_name segmentationID "^prom$"
         if (result)
            call grid_append_tier segmentationID result nucleiID
         endif
   endif

   if (needs_phon_tier and not phones_available)
      call fatal_error Cannot find phoneme tier (named "phon...") in annotation TextGrid
   endif
   if (needs_syll_tier and not syllables_available)
      call fatal_error Cannot find syllable tier (named "syll...") in annotation TextGrid
   endif

   call msg Calculating actual segmentation (Method='segmentation_name$')...
   if (segm_type != segm_extern)
      call make_segmentation segm_type t1s t2s nucleiID
   else
      select segmentationID
      call tier_number_by_name segmentationID "^segm"
      if (result = 0)
         call fatal_error Cannot find segmentation tier (named "segm...") in segmentation TextGrid
      endif
      tier_in = result
      call copy_tier segmentationID tier_in nucleiID nucleus_tier
      select nucleiID
      n_ = Get number of intervals... nucleus_tier
      for j from 1 to n_
         label$ = Get label of interval... nucleus_tier j
         call is_vowel 'label$'
         if (is_vowel)
            Set interval text... nucleus_tier j a
         endif
      endfor
   endif

   call safe_nuclei t1s t2s
   nrof_nuclei_analysed = result

   s$ = "File='basename$' SEG='segmentation_name$' t1='t1s' t2='t2s' GT='glissando' GT_low='glissando_low' ADAPT='adaptive_glissando' DG='diffgt' MINTS='mindur_ts' MINPAUSE='mindur_pause_gap'"
   select nucleiID
   Set interval text... settings_tier 1 's$'

  # prepare for contour analysis
    if (contour_annotation)
       select nucleiID
       Remove tier... contour_tier		; is empty tier when object is created
       if (segm_type == segm_msyllvow or segm_type == segm_msyllpeak or segm_type == segm_mrhyme)	
          ; use syllable boundaries for contour labels
          Duplicate tier... syllable_tier contour_tier contour
       elsif (segm_type == segm_vnucl and segfile_available and syllables_available)
          Duplicate tier... syllable_tier contour_tier contour
       else
          Duplicate tier... nucleus_tier contour_tier contour
       endif
       call tier_clear_text nucleiID contour_tier
    endif ; contour_annotation
    if (stress_annotation)
       if (segm_type == segm_msyllvow or segm_type == segm_msyllpeak or segm_type == segm_mrhyme)
          call tier_replace nucleiID syllable_tier nucleiID stress_tier
       elsif (segm_type == segm_vnucl and segfile_available and syllables_available)
          call tier_replace nucleiID syllable_tier nucleiID stress_tier
       else
          call tier_replace nucleiID nucleus_tier nucleiID stress_tier
       endif
       call tier_clear_text nucleiID stress_tier
    endif ; contour_annotation
   endif ; not segmentation available
endproc


# Clip a PitchTier by replacing data points outside clip range
# by values 1 ST outside range
procedure clipPitchTier objectID ymin ymax xmin xmax
   select objectID
   firsti = Get low index from time... xmin
   if (firsti = 0)
      firsti = 1
   endif
   lasti = Get nearest index from time... xmax
   n = Get number of points
   for i from firsti to lasti
      x = Get time from index... i
      y = Get value at index... i
      if (y > ymax) 
         y = ymax+1
      elsif (y < ymin)
         y = ymin-1
      endif
      Remove point... i
      Add point... x y
   endfor
endproc 


# Delete temporary objects created for current input file
procedure cleanup_current_file
   if (signal_available)
      removeObject (soundID)
      if (volatile)
         removeObject (fullsoundID)
      endif
      signal_available = 0
   endif
   if (intensity_available)
      removeObject (intensityID)
   endif
   if (pitch_available)
      removeObject (pitchID)
   endif
   if (loudness_available)
      removeObject (loudnessID)
   endif
   if (intbp_available)
      removeObject (intbpID)
   endif
   if (inthp_available)
      removeObject (inthpID)
   endif
   if (segfile_available)
      if not boundary_annotation
         removeObject (segmentationID)
      endif
      ;if (draw_prosograms)
         if (nrofplottedtiers > 0)
            removeObject (newgridID)
         endif
      ;endif
   endif
   if (need_stylization or task == task_segmentation)
      removeObject (nucleiID)
   endif
   if (need_stylization)
      # Table with pitch values used in contour_analysis
      if (nucldat_available)
         removeObject (nucldatID)
      endif
      if (stylization_available)
         removeObject (stylID, stylSTID)
      endif
   endif
endproc


procedure cleanup_global
   if (task != task_interactive and task != task_annotation)
      if (need_prosodic_profile)
         filedelete 'globalfile$'
         if (need_prosodic_profile and needs_pitchrange)
            call msg Writing global profile report to "'globalfile$'"
            select reportID
;printline cleanup_global: globalfile=<'globalfile$'>
            Write to headerless spreadsheet file... 'globalfile$'
         endif
         removeObject: reportID
      endif
   endif
   if ((not batch_mode) and variableExists ("filelistID"))
      removeObject: filelistID
   endif
endproc


procedure convert_sampa_ipa objectID tier
   select objectID
   .n = Get number of intervals... tier
   for j from 1 to .n
      label$ = Get label of interval... tier j
      call sampa_ipa
      Set interval text... tier j 'label$'
   endfor
endproc


procedure sampa_ipa
# replace SAMPA label$ by IPA representation in Praat's "special symbols" format
   len = length (label$)
   s$ = ""
   i = 1
   while (i <= len)
      b$ = ""			; translation (i.e. output)
      c1$ = mid$ (label$,i,1)	; next char
      c2$ = mid$ (label$,i,2)	; next 2 chars
      c3$ = mid$ (label$,i,3)	; next 3 chars
      c4$ = mid$ (label$,i,4)	; next 4 chars
      cn$ = mid$ (label$,i+1,1)	; following char
      restlen = len - i
      if (restlen >= 1 and (cn$ = "~" or cn$ = "`" or cn$ = "="))
         if (cn$ = "~")		; nasal vowels (length=2)
            if (c2$ = "a~")
               b$ = "a\~^"
            elsif (c2$ = "A~")
               b$ = "\as\~^"
            elsif (c2$ = "o~")
               b$ = "\o~"
            elsif (c2$ = "E~" or c2$ = "e~") 
               ; b$ = "\ep\~^"
               b$ = "\ef\~^"
            elsif (c2$ = "O~")
               b$ = "\ct\~^"
            elsif (c2$ = "9~")
               b$ = "\oe\~^"
            endif
         elsif (cn$ = "`")	; rhoticity
            if (index ("uoi", c1$))
               b$ = c1$ + "\hr"
            elsif (c2$ = "@`")
               b$ = "\sr"
            elsif (c2$ = "s`")
               b$ = "\s."
            elsif (c2$ = "z`")
               b$ = "\z."
            endif
         elsif (cn$ = "=")	; syllabicity in x-sampa
            if (index_regex (c1$, "[mnNJlrR]")) 
               b$ = c1$ + "\|v"
            endif
         endif
         if (length (b$) > 0)
            s$ = s$ + b$
            i += 2
         else
            s$ = s$ + c1$
            i += 1
         endif
    # SAMPA symbols of length 1
      elsif (index ("SZRMNHJGAEIOQVY@2679?:{}&", c1$))
         if (c1$ = "A") 
            b$ = "\as"
         elsif (c1$ = "E")
            ; b$ = "\ep"
            b$ = "\ef"
         elsif (c1$ = "I")
            b$ = "\ic"
         elsif (c1$ = "O")
            b$ = "\ct"
         elsif (c1$ = "Y")
            b$ = "\yc"
         elsif (c1$ = "Q")
            b$ = "\ab"
         elsif (c1$ = "V")
            b$ = "\vt"
         elsif (c1$ = "2")
            b$ = "\o/"
         elsif (c1$ = "6")
            b$ = "\sr"
         elsif (c1$ = "7") 
            b$ = "\rh"
         elsif (c1$ = "9") 
            b$ = "\oe"
         elsif (c1$ = "@") 
            b$ = "\sw"
         elsif (c1$ = "S") 
            b$ = "\sh"
         elsif (c1$ = "Z") 
            b$ = "\zh"
         elsif (c1$ = "R") 
            b$ = "\rc"
         elsif (c1$ = "M") 
            b$ = "\mj"
         elsif (c1$ = "N") 
            b$ = "\ng"
         elsif (c1$ = "H") 
            b$ = "\ht"
         elsif (c1$ = "J") 
            b$ = "\nj"
         elsif (c1$ = "G") 
            b$ = "\gf"
         elsif (c1$ = "?") 
            b$ = "\?g"
         elsif (c1$ = ":") 
            b$ = "\:f"
         elsif (c1$ = "{") 
            b$ = "\ae"
         elsif (c1$ = "}") 
            b$ = "\u-"
         elsif (c1$ = "&") 
            b$ = "\Oe"
         endif
         if (length (b$) > 0)
            s$ = s$ + b$	; append b$ to output string
            i += 1
         else
            s$ = s$ + c1$
            i += 1
         endif
      elsif (c1$ = "\")
         if (index ("_\:f__\a~__\o~__\o/__\ef_", "_'c3$'_"))
            s$ = s$ + c3$
            i += 3
;         elsif (restlen >= 3 and index (":\a~:\o~:\o/:", ":'c3$':") > 0)
;            s$ = s$ + c3$
;            i += 3
         elsif (restlen >= 4 and index (":\ep~:", c4$) > 0)
            s$ = s$ + c4$
            i += 4
         else			; others: just copy them
            s$ = s$ + c1$
            i += 1
         endif
      elsif (c3$ = "i_d")
         s$ = s$ + "i\Nv"
         i += 3
      else			; others: just copy them
         s$ = s$ + c1$
         i += 1
      endif
   endwhile
   label$ = s$
endproc


procedure speaker_info_get
# 1. Make list of speakers appearing in tier "speaker" of annotated segmentation textgrid.
# 2. Add speaker identification to nucldatID, as a number (in column <speaker_id>) and as a string (in row label).
   speakers = 0			; nrof different speakers found
   speakers$ = ""		; used to store speaker names and their number
   if (speaker_available)
       speaker = 0		; nrof current speaker
       select nucldatID
       rows = Get number of rows
       for row to rows
          t1 = Get value... row nucleusstarttime
          t2 = Get value... row nucleusendtime
          select nucleiID
          i = Get interval at time... speaker_tier t1+(t2-t1)/2
          speaker$ = Get label of interval... speaker_tier i
          speaker$ = replace_regex$ (speaker$, "^ +(.*) +$", "\1", 1)	; trim left and right
          if (index (speakers$, "<'speaker$'>") > 0)			; speaker already encountered in file
             speaker = extractNumber (speakers$, "<'speaker$'>:")	; get his number
          else								; new speaker
             speakers += 1	; found new speaker
             speaker = speakers	; its number
             speakers$ = speakers$ + "<'speaker$'>:'speaker' "	; store name and number in list
             speaker_label'speaker'$ = speaker$
          endif
          select nucldatID
          Set value... row speaker_id speaker
          ; speaker$ = replace_regex$ (speaker$, " ", "_", 0)
          if (length(speaker$) < 1)
             ; speaker$ = "_"
          endif
          Set row label (index)... row "'speaker$'"
       endfor
   else
      call msg Input textgrid does not contain speaker tier. Assuming there's 1 speaker.
      speakers = 1
      speaker$ = "ANON"
      speaker_label'speakers'$ = speaker$
      speakers$ = "<'speaker$'>:'speakers' "
      select nucleiID
      Set interval text... speaker_tier 1 'speaker$'
      select nucldatID
      rows = Get number of rows
      for row from 1 to rows
         Set value... row speaker_id 1
      endfor
   endif
endproc


procedure speaker_autorange t1 t2
; propose a pitch range suitable for plotting the range of all speakers in time interval <t1>..<t2>
; return range in ST in <ymin>..<ymax>
   if (task == task_pitch_plot or not needs_pitchrange)	; no nuclei availale
      select pitchID
      y_ = Get mean... t1 t2 semitones re 1 Hz
      if (y_ == undefined)
         call msg speaker_autorange: mean pitch undefined for time 't1:3'-'t2:3', using default 85 ST
         y_ = 85
      endif
      ymin = y_ - 12
      ymax = y_ + 12
   else ; compute total range for all speakers in time interval <t1>..<t2>
      ymin = 1000 
      ymax = 0
      t_ = t1
      select nucleiID
      .n = Get number of intervals... speaker_tier
      t2n = Get end point... speaker_tier .n
      t2_ = min (t2, t2n)
      repeat		; all speakers in range t1..t2
         i = Get interval at time... speaker_tier t_
         speaker$ = Get label of interval... speaker_tier i
         speaker$ = replace_regex$ (speaker$, "^ +(.*) +$", "\1", 1)	; trim left and right
         speaker_j = extractNumber (speakers$, "<'speaker$'>:")		; get his number
;;call msg speaker_autorange: t='t_:3' t2='t2_:3' i='i' speaker_j='speaker_j' speaker='speaker$'
         if (speaker_j == 0 or speaker_j == undefined)
            call msg speaker_autorange: speaker undefined at time 't_:3'
            ; This happens when there are no nuclei for this speaker 
            select pitchID
            ymin = Get mean... 0 0 semitones re 1 Hz
            ymax = ymin
         else
            y_ = extractNumber (speaker_range_'speaker_j'$, "BOTTOM_ST=")
            if (y_ == undefined)
               call msg speaker_autorange: BOTTOM_ST undefined t='t_:3'
               y_ = 85
            endif
            ymin = min (ymin, y_) 
            y_ = extractNumber (speaker_range_'speaker_j'$, "TOP_ST=")
            if (y_ == undefined)
               call msg speaker_autorange: TOP_ST undefined t='t_:3'
               y_ = 85
            endif
            ymax = max (ymax, y_)
         endif
         select nucleiID
         t_ = Get end point... speaker_tier i
      until (t_ >= t2_)
      if (ymax - ymin < 24)		; default range 24 ST or 2 octaves 
         ymin = max(0, (ymin + (ymax - ymin)/2) - 12)
         ymax = ymin + 24
      endif
   endif ; task
endproc


procedure nucleus_tier_postproc grid_ phon_tier_ syll_tier_
   select grid_
   interval_tier = Is interval tier... phon_tier_
   if (interval_tier)
      Set tier name... phon_tier_ x
      n_ = Get number of intervals... phon_tier_
      ; replace parts other than nucleus by zero-length strings
      for i_ to n_
         s_$ = Get label of interval... phon_tier_ i_
         s_$ = ":'s_$':"
         if (index (":<:>:<>:xL:xR:U:skip:reject:short:", s_$) > 0)
            s_$ = ""
            Set interval text... phon_tier_ i_ 's_$'
         endif
      endfor
      ; reduce contiguous empty intervals
      i_ = 2
      while (i_ <= n_)
         prev_$ = Get label of interval... phon_tier_ i_-1
         s_$ = Get label of interval... phon_tier_ i_
         if (prev_$ = "" and s_$ = "")
            Remove left boundary... phon_tier_ i_
            n_ -= 1
         else
            i_ += 1
         endif
      endwhile
      ; add syllable boundaries to nucleux tier
      empty$ = ""
      n_ = Get number of intervals... syll_tier_
      for i_ to n_ - 1
         t_ = Get end point... syll_tier_ i_
         j_ = Get interval at time... phon_tier_ t_
         x1_ = Get start point... phon_tier_ j_
         x2_ = Get end point... phon_tier_ j_
         if (t_ - x1_ > time_step and t_ < x2_)
            Insert boundary... phon_tier_ t_
         endif
         Set interval text... syll_tier_ i_ 'empty$'
      endfor
   endif
endproc


procedure copy_tier_tweeked srcgrid srctier destgrid desttier
; Special treatment for Rhapsodie tier "locuteur". Force interval contiguity.
      select destgrid
      .n = Get number of intervals... desttier
      .endtime = Get end point... desttier .n 
      select srcgrid
      .n = Get number of intervals... srctier
      for .i to .n
         select srcgrid
         .t1 = Get start point... srctier .i
         .t2 = Get end point... srctier .i
         .label$ = Get label of interval... srctier .i
         if (.i < .n)
            .t3 = Get start point... srctier .i+1
            if (.t2 < .t3)
               .t2 = .t3	; intervals should be contiguous
            endif
         endif
         select destgrid
         if (.t2 < .endtime and .i < .n)
            Insert boundary... desttier .t2
         endif
         if (.t1 <= .endtime)
            .t2 = min (.t2, .endtime)	; when destgrid is shorter than srcgrid
            j = Get interval at time... desttier (.t1 + (.t2-.t1)/2)
            Set interval text... desttier j '.label$'
         endif
      endfor
endproc


procedure validate_syllable_tier infname_$
; Validate (1) symbols in phon tier, (2) symbols in syllable tier and (3) alignment of syllable and phoneme tiers.
; (4) Check number of vowels (or syllabic consonants) within syllable.
; Uses tiers named "phon...", "syll...", and optionally "word...".
; Test is skipped when syllable label equals "_" or when word label equals "%".
   call fname_parts 'infname_$'
   basename_$ = result2$
   dataID = Read from file... 'infname_$'
   call corpus_conversion 1 "dataID" "'corpus$'" 'contourfile$'
   call tier_get dataID "^phon" phone_tier "Cannot find phoneme tier in 'basename_$'" 1
   call tier_get dataID "^syll" syllable_tier "Cannot find syllable tier in 'basename_$'" 1
   call tier_get dataID "^word" word_tier "Cannot find word tier in 'basename_$'" 0

   .ns = Get number of intervals... syllable_tier
   for j from 1 to .ns
      .t1_syll = Get start point... syllable_tier j
      .t2_syll = Get end point... syllable_tier j
      .label_syll$ = Get label of interval... syllable_tier j
      if (word_tier)
         .k = Get interval at time... word_tier (.t1_syll+(.t2_syll-.t1_syll)/2)
         word_$ = Get label of interval... word_tier .k
         word_$ = replace_regex$ (word_$, " ", "", 0)	; rm whitespace
      else
         word_$ = ""
      endif
      if (index("_#", .label_syll$)== 0 and word_$ <> "%")	; not pause, not noise, not incomprehensible
         errors = 0
         ; Check label in tier
            call verify_tier "'.label_syll$'" "'basename_$'" "syllable" .t1_syll
            errors += result
         call interval_from_time dataID phone_tier .t1_syll ph1
         call interval_from_time dataID phone_tier .t2_syll-0.001 ph2
         nrof_vowels = 0
         nrof_syllabics = 0
         for .phon from ph1 to ph2
            .t1_phon = Get start point... phone_tier .phon
            .t2_phon = Get end point... phone_tier .phon
            .label_phon$ = Get label of interval... phone_tier .phon
            call verify_tier "'.label_phon$'" "'basename_$'" "phoneme" .t1_phon
            errors += result
            ; Check alignment between syllable tier and phoneme tier
            if (.phon == ph1 and .t1_phon <> .t1_syll)
              call msg Syllable ('.label_syll$') starting at '.t1_syll:4' not aligned with phoneme ('.label_phon$') start ('.t1_phon:4')
              errors += 1
            endif
            if (.phon == ph2 and .t2_phon <> .t2_syll)
              call msg Syllable ('.label_syll$') ending at '.t2_syll:4' not aligned with phoneme ('.label_phon$') end ('.t2_phon:4')
              errors += 1
            endif
            ; Check number of vowels/syllabics
            call is_syllabic '.label_phon$'
            if (result)
               nrof_syllabics += 1
               if (is_vowel)
                  nrof_vowels += 1
               endif
               if (nrof_syllabics > 1)
                 call msg Multiple vowels (or syllabics) in syllable ('.label_syll$') at time '.t1_syll:4'
               endif
            endif ; syllabic
            if (.phon == ph2 and nrof_vowels == 0)
               call msg No vowel in syllable ('.label_syll$') at time '.t1_syll:4'
            endif
         endfor ; phonemes in syllable
      endif
   endfor ; syllable j
   Remove
endproc


procedure verify_tier .str$ fname_$ .type$ time_
   result = 0
   if (index(.str$, " "))
      printline ERROR in 'fname_$' in tier 'tier_' at time 'time_:2': contains whitespace
   endif
   if (.type$ = "phoneme")
      .str$ = replace$(.str$, "\rc", "R", 0)
      .str$ = replace$(.str$, "\ep", "E", 0)
      if (not index_regex(.str$, "^[ptkbdgfsSvzZxhmnNJlRiyue2oE9OajwH@\?][=~]?$"))
         call msg Label in phone tier at time 'time_:2' contains invalid character: <'.str$'>
         result = 1
      endif
   elsif (.type$ = "syllable")
      .str$ = replace$(.str$, "\rc", "R", 0)
      .str$ = replace$(.str$, "\ep", "E", 0)
      if (not index_regex(.str$, "^[ptkbdgfsSvzZxhmnNJlRiyue2oE9OajwH@\?=~]+$"))
         call msg Label in syllable tier at time 'time_:2' contains invalid character: <'.str$'>
         result = 1
      endif
   elsif (.type$ = "hesitation")
      if (not index_regex(.str$, "^H?$"))
         call msg Label in hesitation tier at time 'time_:2' contains invalid character: <'.str$'>
         result = 1
      endif
   elsif (.type$ = "prominence")
      if (not index_regex(.str$, "^[0SW]$"))
         call msg Label in prominence tier at time 'time_:2' contains invalid character: <'.str$'>
         result = 1
      endif
   elsif (.type$ = "contour")
      if (not index_regex(.str$, "^[BLMHT_RrFfSC,]*$"))
         call msg Label in contour tier at time 'time_:2' contains invalid sequence: <'.str$'>
         result = 1
      endif
      if (index_regex(.str$, "CC") or index_regex(.str$, "__"))
         call msg Label in contour tier at time 'time_:2' contains invalid sequence: <'.str$'>
         result = 1
      endif
   elsif (.type$ = "stressref")
      if (not index_regex(.str$, "^[SW0]$"))
         call msg Label in stress-ref tier at time 'time_:2' contains invalid character: <'.str$'>
         result = 1
      endif
   else
      call msg verify_tier: ERROR invalid tier type: <'.type$'>
   endif
endproc


procedure store_stylization paramID nucleiID .filename$
# Store stylization targets, indicating whether they are initial/final/intermediate relative to nucleus boundaries
   filedelete '.filename$'
   select nucleiID						; segmentation into nuclei, etc
   .nnucl = Get number of intervals... nucleus_tier
   select paramID						; stylization (PitchTier)
   .ni = Get number of points
   for .interv from 1 to .nnucl					; indices of nuclei for which to plot stylization 
      select nucleiID						; segmentation into nuclei, etc
      .label$ = Get label of interval... nucleus_tier .interv
      if (.label$ = "a")						; it's a nucleus
         .nx1 = Get start point... nucleus_tier .interv		; time of start of nucleus
         .nx2 = Get end point... nucleus_tier .interv		; time of end of nucleus
         select paramID						; stylization (PitchTier)
       # Check that the nearest indices are within nucleus
         .i1 = Get nearest index from time... .nx1
         repeat
            .t1 = Get time from index... .i1
            if (.t1 < .nx1)
               .i1 += 1
            endif
         until (.t1 >= .nx1 or .t1 >= .nx2 or .i1 >= .ni)
         .i2 = Get nearest index from time... .nx2
         repeat
            .t2 = Get time from index... .i2
            if (.t2 > .nx2)
               .i2 -= 1
            endif
         until (.t2 <= .nx2 or .t2 <= .nx1 or .i2 <= 1)
         for .i from .i1 to .i2					; each stylization (tonal)  segment i
            select paramID					; stylization (PitchTier)
            .x = Get time from index... .i
            .y = Get value at index... .i
            .yST =  hertzToSemitones(.y) - hertzToSemitones(1)
            if (.i == .i1)
               if (syllables_available)				; write syllable start, end and label
                  select nucleiID
                  .isyll = Get interval at time... syllable_tier .x
                  .t1s = Get start point... syllable_tier .isyll
                  .t2s = Get end point... syllable_tier .isyll
                  .s$ = Get label of interval... syllable_tier .isyll
                  fileappend '.filename$' syll'tab$''.t1s:4''tab$''.t2s:4''tab$''.s$''newline$'
               endif
               fileappend '.filename$' nucl'tab$''.nx1:4''tab$''.nx2:4''newline$'
               .type$ = "tstart"	; target at start of nucleus
            elsif (.i == .i2)
               .type$ = "tend"	; target at end of nucleus
               ; fileappend '.filename$' Ne'tab$''.nx2:4''newline$'
            else
               .type$ = "tin"	; target inside nucleus
            endif
            fileappend '.filename$' '.type$''tab$''.x:4''tab$''.y:1''tab$''.yST:1''newline$'
         endfor
      endif
   endfor
endproc


procedure store_stylization_resampled paramID nucleiID .dx .filename$
# Store resampled stylization, as if it were a pitch curve, sampled at .dx (second)
# <nucleiID>	segmentation into nuclei, etc
   filedelete '.filename$'
   select paramID						; stylization (PitchTier)
   .np = Get number of points
 ; nrof targets in Pitch Tier
   .starttime = Get start time
   .endtime = Get end time
   .nx = floor((.endtime-.starttime)/.dx)			; nrof pitch samples in Pitch object
   fileappend '.filename$' File type = "ooTextFile"'newline$'
   fileappend '.filename$' Object class = "Pitch 1"'newline$''newline$'
   fileappend '.filename$' xmin = '.starttime''newline$'
   fileappend '.filename$' xmax = '.endtime''newline$'
   fileappend '.filename$' nx = '.nx''newline$'
   fileappend '.filename$' dx = '.dx''newline$'
   fileappend '.filename$' x1 = '.starttime''newline$'
   fileappend '.filename$' ceiling = 600'newline$'
   fileappend '.filename$' maxnCandidates = 1'newline$'
   fileappend '.filename$' frame []:'newline$'
   .i = 0							; frame index
   .t = .starttime
   while (.t < .endtime)
      .i += 1							; frame index for current time
      .value = 0						; default F0 (outside nuclei)
      select nucleiID						; segmentation into nuclei, etc
      .interv = Get interval at time... nucleus_tier .t
      .label$ = Get label of interval... nucleus_tier .interv
      .nx1 = Get starting point... nucleus_tier .interv		; time of start of nucleus
      .nx2 = Get end point... nucleus_tier .interv		; time of end of nucleus
      select paramID						; stylization (PitchTier)
      .i1 = Get low index from time... .t			; index of point in pitch tier >= .t
      if (not .i1 == undefined) and (.i1 > 0) 
         .t1 = Get time from index... .i1
         if (.i1 < .np)
            .i2 = .i1 + 1					; index of next point in pitch tier
            .t2 = Get time from index... .i2			; time of next point
         else
            .i2 = .i1
            .t2 = .nx2
         endif
         if (.label$ = "a" and .t >= .nx1 and .t <= .nx2 and .t2 <= .nx2)
            .y1 = Get value at index... .i1
            .y2 = Get value at index... .i2
            .value = .y1 + ((.y2-.y1)*(.t-.t1)/(.t2-.t1)) 
;fileappend '.filename$' ****** t='.t:4' i1='.i1' i2='.i2' t1='.t1:4' 'newline$'
;fileappend '.filename$' ****** t1='.t1:4' y1='.y1:1' t2='.t2:4' y2='.y2:1' v='.value:1' 'newline$'
         endif
      endif
      fileappend '.filename$' 'tab$'frame ['.i']:'newline$'
      fileappend '.filename$' 'tab$''tab$'intensity = 0'newline$'
      fileappend '.filename$' 'tab$''tab$'nCandidates = 1'newline$'
      fileappend '.filename$' 'tab$''tab$'candidate = []'newline$'
      fileappend '.filename$' 'tab$''tab$''tab$'candidate = [1]:'newline$'
      fileappend '.filename$' 'tab$''tab$''tab$''tab$'frequency = '.value:0''newline$'
      fileappend '.filename$' 'tab$''tab$''tab$''tab$'strength = 1'newline$'
      .t += .dx
   endwhile
endproc


procedure option_get .type$ .optname$ .varname$ .default$ .data$
# Extract an option value from a command string.
# <.type>	option type: {word|real|boolean} 
#			word: a string without whitespace
#			line: a string with whitespace
#			real: a number
#			boolean: a boolean where TRUE is encoded by one of: y, Y, yes, 1, T, TRUE
# <.optname>	option name, e.g. "file=", "threshold="
# <.varname>	name of variable, where value of option will be stored
# <.default>	default option value, when option is not mentioned in command
# <.data>	command argument string, where options will be searched
   if (.type$ = "word")
      .s$ = extractWord$ (.data$, .optname$)
      '.varname$'$ = .default$
      if (length (.s$))
         '.varname$'$ = .s$
      endif
   elsif (.type$ = "line")
      .s$ = extractLine$ (.data$, .optname$)
      '.varname$'$ = .default$
      if (length (.s$))
         '.varname$'$ = .s$
      endif
   elsif (.type$ = "real")
      .v = extractNumber (.data$, .optname$)
      '.varname$' = '.default$'
      if not (.v == undefined)
         '.varname$' = .v
      endif
   elsif (.type$ = "boolean")
      .s$ = extractWord$ (.data$, .optname$)
      '.varname$' = 0		; default value
      if (length (.s$) < 1)
         .s$ = .default$
      endif
      if (index (":y:1:Y:yes:T:TRUE:", ":'.s$':") > 0)
         '.varname$' = 1
      endif
   endif
endproc


procedure option_check .type$ .optname$ .valids$ .value$
# Check whether the value of an option belongs to a set of accepted values
   if (.type$ = "word")
      if (index (.valids$, ":'.value$':") <= 0)
         call msg invalid value <'.value$'> for option <'.optname$'>
      endif
   endif
endproc


procedure calculate_pitch .cmda$
   call option_get word file= inputfname "" '.cmda$'
   call option_get real time_step= time_step 0.005 '.cmda$'
   call option_get real f0min= minimum_pitch 60 '.cmda$'
   call option_get real f0max= maximum_pitch 450 '.cmda$'
   if (variableExists ("corpus$") and length (corpus$) and variableExists ("corpus_home$"))
      if (variableExists ("corpus_subdir_sound$"))
         inputfname$ = corpus_home$ + corpus_subdir_sound$ + inputfname$
      else
         inputfname$ = corpus_home$ + inputfname$
      endif
   endif
   call files_get_regex "filelistID" 'inputfname$'
   select filelistID
   nrofFiles = Get number of strings
   if (nrofFiles == 0)
      call msg No input files found for <'inputfname$'>
   endif
   task = task_calc_pitch
   for iFile to nrofFiles
      select filelistID
      fname$ = Get string... iFile
      call initialization_per_file
      call construct_filenames 'fname$'
      call calculate_pitch_int
      removeObject: soundID, pitchID
   endfor
   removeObject: filelistID
endproc


# arguments: name, default
#	file=		""
#	time_step=	0.005
#	f0min=		60
#	f0max=		450
#	t1=			0
#	t2=			0
#	time_incr=	3.0
#	segmentation=	rhyme
#	dmin=		0.035
#	draw=		yes
#	wide=		yes
#	rich=		yes
#	pitchrange=	yes
#	tiers=		*1,2,3	; tiers shown in the graphics output
#	collect=	no		; collect output of duration calculation in a single output file
#	save=		no		; save intermediate data in files

procedure prosogram_variants .cmda$
; call msg prosogram_variants: starts <'.cmda$'>
   ; output_filename$ = "<input_directory>/<basename>_"
   adaptive_glissando = 1
   glissando = 0.32
   glissando_low = 0.16
   diffgt = 30
   auto_pitchrange = 1		; automatic pitch range adjustment
   draw_pitch_target_values = 0
   outputmode$ = "Fill"
   output_format$ = "EPS"
   need_prosodic_profile = 0
   call option_get word file= inputfname "" '.cmda$'
   call option_get real time_step= time_step 0.005 '.cmda$'
   call option_get real f0min= minimum_pitch 60 '.cmda$'
   call option_get real f0max= maximum_pitch 450 '.cmda$'
   call option_get real t1= pmf_t1 0 '.cmda$'
   call option_get real t2= pmf_t2 0 '.cmda$'
   call option_get real time_incr= timeincr 3.0 '.cmda$'
   call option_get word segmentation= segmentation_name rhyme '.cmda$'
   call option_check word segmentation :vow-nucl:extern:loudness:int-BP:asyll:rhyme:syll+vow:syll:voiced: 'segmentation_name$'
   call option_get real dmin= mindur_ts 0.035 '.cmda$'
   call option_get boolean draw= draw_prosograms_switch yes '.cmda$'
   call option_get boolean wide= wide yes '.cmda$'
   call option_get boolean rich= rich yes '.cmda$'
   call option_get boolean pitchrange= show_pitchrange no '.cmda$'
   call option_get word tiers= tiers_to_show *1,2,3 '.cmda$'
   call option_get boolean collect= collect_output no '.cmda$'
   call option_get word outputfile= outputfname "" '.cmda$'
   call option_get boolean save= save_intermediate_data no '.cmda$'
   view = 2
   if (wide)
      view = 4
   endif
   if (variableExists ("corpus$") and length (corpus$) and variableExists ("corpus_home$"))
      if (variableExists ("corpus_subdir_sound$"))
         inputfname$ = corpus_home$ + corpus_subdir_sound$ + inputfname$
      else
         inputfname$ = corpus_home$ + inputfname$
      endif
   endif

   ; call msg prosogram_variants: 'inputfname$' t1='pmf_t1' t2='pmf_t2' time_step='time_step:3' segmentation_name='segmentation_name$' wide='wide' rich='rich' pitchrange='show_pitchrange' adapative='adaptive_glissando'
   ; call msg prosogram_variants: 'inputfname$' t1='pmf_t1' t2='pmf_t2' time_step='time_step:3' time_incr='timeincr:3' outputfname='outputfname$'
   call msg prosogram_variants: save_intermediate_data='save_intermediate_data'

   call files_get_regex "filelistID" 'inputfname$'
   select filelistID
   nrofFiles = Get number of strings
   if (nrofFiles == 0)
      call msg No input files found for <'inputfname$'>
   endif
   task = task_prosogram
   if (contour_annotation)
      task = task_autoanno
   endif
   call task_flags
   draw_prosograms = draw_prosograms_switch
   call initialization_multiple_files
   for iFile to nrofFiles
      select filelistID
      fname$ = Get string... iFile
      call initialization_per_file
      call construct_filenames 'fname$'
      call process_one_input_file pmf_t1 pmf_t2 'fname$'
   endfor
   call cleanup_global
   removeObject: filelistID
   call msg Batch command prosogram: Ready
endproc


procedure prosogram .cmda$
   contour_annotation = 0
   boundary_annotation = 0
   call prosogram_variants '.cmda$'
endproc


procedure tonal_annotation .cmda$
   contour_annotation = 1
   boundary_annotation = 0
   call prosogram_variants '.cmda$'
   contour_annotation = 0
   call msg Batch command tonal_annotation: Ready
endproc


procedure detect_boundaries .cmda$
   boundary_annotation = 1
   contour_annotation = 0
   call prosogram_variants '.cmda$'
   boundary_annotation = 0
endproc


procedure calculate_duration_data .cmda$
; Specify the filename as wav files, because of the wildcard limitations of "Create Strings as file list..."
; Results of all selected input file are collected in 1 output file
   call option_get word file= filespec "" '.cmda$'
   call option_get word unit= unit "sound" '.cmda$'
   ; Collect output = collect results of multiple input files in one output file
   call option_get boolean collect= collect_output yes '.cmda$'
   call fname_parts 'filespec$'
   indir$ = result4$
   .s$ = indir$ + "_dur_'unit$'.txt"
   call option_get word out= outfname "'.s$'" '.cmda$'
   Create Strings as file list... filelist 'filespec$'
   .nfiles = Get number of strings
   call msg calculate_duration_data: filespec='filespec$', '.nfiles' files, collect='collect_output', outfname='outfname$'
   for .j to .nfiles
      select Strings filelist
      fname$ = Get string... .j
      call fname_parts 'fname$'
      basename$ = result2$
      fname$ = indir$ + basename$ + ".TextGrid"
      contourfile$ = indir$ + basename$ + "_contour.TextGrid"
      call msg file='.j'  <'fname$'> out='outfname$' contour='contourfile$'
      dataID = Read from file... 'fname$'
      call corpus_conversion 1 "dataID" "'corpus$'" 'contourfile$'
      if (unit$ = "sound")
         .f = .j
         if (not collect_output)
            .f = 1
         endif
         call duration_data_from_textgrid dataID .f "'unit$'" "'outfname$'" 'basename$'
      endif
      removeObject: dataID
   endfor
   select Strings filelist
   Remove
   call msg Batch command Ready
endproc


procedure convert_eps .cmda$
# Assumes Ghostscript, nconvert and pdftk are installed.
# nconvert is required when converting to GIF format. http://www.xnview.com/
# pdftk is required when creating multipage PDF files. http://www.pdflabs.com/tools/pdttk-the-pdf-toolkit/
; Specify the filename as eps files
   call option_get word file= filespec "*.eps" '.cmda$'
   call fname_parts 'filespec$'
   basename$ = result2$
   indir$ = result4$
   call option_get word outfname= outfname "'basename$'.pdf" '.cmda$'
   call option_get word format= graphics_format "JPG" '.cmda$'
   call option_check word graphics_format :PNG:GIF:JPG:PDF:multipage_PDF: 'graphics_format$'
   call option_get word resolution= resolution "300" '.cmda$'
   call option_check word resolution :72:96:120:300:600: 'resolution$'
   call option_get word outdir= outdir "<same_as_input>" '.cmda$'
   if (outdir$ = "<same_as_input>")
      outdir$ = indir$
   else
      call fname_parts 'outdir$'
      outdir$ = result4$
   endif
   Create Strings as file list... filelist 'filespec$'
   .nfiles = Get number of strings
   pdf_files$ = ""
   call msg convert_eps: filespec='filespec$', '.nfiles' files, outdir='outdir$'
   call msg convert_eps: format='graphics_format$'
   eps_pdf$ = path_ghostscript$ + " -dNOPAUSE -dBATCH -dQUIET -sDEVICE=pdfwrite"
   eps_jpg$ = path_ghostscript$ + " -dNOPAUSE -dBATCH -dQUIET -sDEVICE=jpeg -sEPSCrop"
   eps_png$ = path_ghostscript$ + " -dNOPAUSE -dBATCH -dQUIET -sDEVICE=png16 -sEPSCrop"
   png_gif$ = path_nconvert$
   for .j to .nfiles
      select Strings filelist
      fname$ = Get string... .j
      call fname_parts 'fname$'
      basename$ = result2$
      pdf_files$ = pdf_files$ + "'basename$'.pdf "
      src$ = indir$ + basename$
      dst$ = outdir$ + basename$
      res = 'resolution$'
;printline basename='basename$' src='src$'
      if (graphics_format$ = "GIF")
         system 'eps_png$' -r'res' -sOutputFile="'dst$'.png" "'src$'.eps"
         system 'png_gif$' -quiet -out gif 'dst$'.png
         filedelete 'dst$'.png
      elsif (graphics_format$ = "PNG")
         system 'eps_png$' -r'res' -sOutputFile="'dst$'.png" "'src$'.eps"
      elsif (index (graphics_format$, "PDF"))	; PDF or multipage PDF
         if (res == 96 or res == 120)
            call msg Invalid resolution for PDF. Using 300 dpi.
            res = 300
         endif
;printline command= system 'eps_pdf$' -r'res' -sOutputFile="'dst$'.pdf" "'src$'.eps"
         system 'eps_pdf$' -r'res' -sOutputFile="'dst$'.pdf" "'src$'.eps"
      elsif (graphics_format$ = "JPG")
         if (res == 96 or res == 120)
            call msg Invalid resolution for JPEG. Using 300 dpi.
         endif
         system 'eps_jpg$' -r'res' -sOutputFile="'dst$'.jpg" "'src$'.eps"
      endif
   endfor
   if (index (graphics_format$, "multipage"))
; use wildcard expansion by pdftk in order to avoid command line buffer overflow
      ;pdf_out$ = replace_regex$ (filespec$, "[0-9\*]*\.eps", "\.pdf", 1)
      pdf_out$ = outdir$ + "/" + outfname$
      pdf_out$ = replace$ (pdf_out$, "//", "/", 0)
      pdf_in$ = replace_regex$ (filespec$, "\.eps", "\.pdf", 1)
printline convert_eps: system pdftk 'pdf_in$' cat output 'pdf_out$'
      system pdftk 'pdf_in$' cat output 'pdf_out$'
   endif
   select Strings filelist
   Remove
   call msg Batch command convert_eps Ready
endproc


procedure corpus_open .cmda$
; Load signal files and corresponding TextGrid files. Specify filenames as wav files.
   call option_get word file= filespec "*.wav" '.cmda$'
   call fname_parts 'filespec$'
   indir$ = result4$
   Create Strings as file list... filelist 'filespec$'
   .nfiles = Get number of strings
   for .j to .nfiles
      select Strings filelist
      fname$ = Get string... .j
      call fname_parts 'fname$'
      basename$ = result2$
      Open long sound file... 'indir$''fname$'
      .dur = Get total duration
      call msg 'basename$''tab$''.dur:3'
      .tg$ = indir$ + basename$ + ".TextGrid"
      if (fileReadable (.tg$))
         Read from file... '.tg$'
      endif
   endfor
   select Strings filelist
   Remove
endproc


procedure textgrid_merge .cmda$
; Merge pairs of TextGrid files.
   call option_get word file1= filespec "*.TextGrid" '.cmda$'
   call option_get word file2_dir= twin_dir "" '.cmda$'
   call option_get word file2_suffix= twin_suffix "" '.cmda$'
   call option_get word out_dir= out_dir "" '.cmda$'
   call option_get word out_suffix= out_suffix "" '.cmda$'
   call option_get boolean convert= convert no '.cmda$'
call msg textgrid_merge: twin_suffix=<'twin_suffix$'>
   call files_get_regex "filelistID" 'filespec$'
   select filelistID
   .nfiles = Get number of strings
   if (.nfiles == 0)
      call msg No input files found for <'filespec$'>
   endif
   for .j to .nfiles
      select filelistID
      fname$ = Get string: .j
      call fname_parts 'fname$'
      basename$ = result2$
      .tg1 = Read from file: fname$
      .dur = Get total duration
      call msg 'newline$'File: 'basename$''tab$''.dur:3'
      .tg$ = twin_dir$ + basename$ + twin_suffix$ + ".TextGrid"
      .out$ = out_dir$ + basename$ + out_suffix$ + ".TextGrid"
      ;call msg textgrid_merge: OutFile: '.out$'
      if (not fileReadable (.tg$))
         call msg File not found: '.tg$'
         removeObject: .tg1
      else
         if (convert)
            ;call process_cprom .tg1
         endif
         .tg2 = Read from file: .tg$
         selectObject: .tg1, .tg2
         .tg3 = Merge
         if (convert)
            call tier_number_by_name .tg3 "^contour$"
            if (result)
               Set tier name... result polytonia
            endif
         endif
         Save as text file: .out$
         removeObject: .tg1, .tg2, .tg3
      endif
   endfor
   removeObject: filelistID
   call msg Batch command textgrid_merge: Ready
endproc


procedure textgrid_prune .cmda$
; Prune tiers in TextGrid files.
   call option_get word file= filespec "*.TextGrid" '.cmda$'
   call option_get word keep= keep "" '.cmda$'
   call option_get word out_dir= out_dir "" '.cmda$'
   call option_get word out_suffix= out_suffix "" '.cmda$'
   call option_get boolean convert= convert no '.cmda$'
   call files_get_regex "filelistID" 'filespec$'
   select filelistID
   .nfiles = Get number of strings
   if (.nfiles == 0)
      call msg No input files found for <'filespec$'>
   endif
   for .j to .nfiles
      select filelistID
      .fname$ = Get string: .j
      call msg 'newline$'File: '.fname$'
      call fname_parts '.fname$'
      .basename$ = result2$
      .tgID = Read from file: .fname$
      .n = Get number of tiers
      .set$ = ",'keep$',"
      .t = .n
      while (.t > 0)
         .tier$ = Get tier name: .t
         if (index (.set$, ",'.tier$',") = 0)
            Remove tier: .t
         endif
         .t -= 1
      endwhile
      .out$ = out_dir$ + .basename$ + ".TextGrid"
      Save as text file: .out$
      removeObject: .tgID
   endfor
   removeObject: filelistID
   call msg Batch command textgrid_prune: Ready
endproc


procedure count_labels .cmda$
; Count occurrences of distinct labels in TextGrid files.
; The tier may be specified by name or number (useful when tier name contains whitespace)
   call option_get word file= filespec "*.TextGrid" '.cmda$'
   call option_get word tier= tiername "phon" '.cmda$'
   .tier = number (tiername$)			; will be undefined when tiername is not a number
   call fname_parts 'filespec$'
   .basename$ = result2$
   if (variableExists ("corpus$"))
      if (length (corpus$))
         if (variableExists ("corpus_subdir_tg$"))
            filespec$ = corpus_home$ + corpus_subdir_tg$ + .basename$ + ".TextGrid"
         endif
      endif
   endif
   ; call msg filespec='filespec$'
   call files_get_regex "filelistID" 'filespec$'
   select filelistID
   .nfiles = Get number of strings
   if (.nfiles == 0)
      call msg count_labels: No input files found for <'filespec$'>
   endif
   call fname_parts 'filespec$'
   .indir$ = result4$
   .ndistinctlabels = 0
   .labels$ = ""
   for .jfile to .nfiles
      select filelistID
      .tg$ = Get string... .jfile
      call msg FILE='.tg$'
      if (fileReadable (.tg$))
         .tg = Read from file: .tg$
         if (.tier = undefined)
            call tier_number_by_name .tg "'tiername$'"
            .tier = result
         endif
         if (.tier)
            .ni = Get number of intervals... .tier
            for .i to .ni
               e$ = Get label of interval... .tier .i
               if (length (e$))
                  if (index (.labels$, "	'e$'	:"))
                     .j = extractNumber (.labels$, "	'e$'	:") 
                     .count_'.j' = .count_'.j' + 1
                  else
                     .ndistinctlabels += 1
                     .labels$ = .labels$ + "	'e$'	:'.ndistinctlabels'"
                     .count_'.ndistinctlabels' = 1
                     .label_'.ndistinctlabels'$ = e$
                  endif
               endif ; length
            endfor ; interval
         endif
         removeObject: .tg
      endif
   endfor
   call msg LABEL	COUNT
   for .j to .ndistinctlabels
      .s$ = .label_'.j'$
      .n = .count_'.j'
      call msg '.s$'	'.n'
   endfor
   removeObject: filelistID
   call msg Batch command count_labels Ready
endproc


procedure find_nucleus .t1 .t2
; Find valid nucleus in time range <.t1>..<.t2>
; Returns index of nucleus (in nucleus_tier) in variable <result> or 0 if not found
   result = 0
   select nucleiID
   .endt = Get end time
   .endt = min (.endt, .t2)
   while (.t1 < .endt and result == 0)
     .i = Get interval at time... nucleus_tier .t1
     .s$ = Get label of interval... nucleus_tier .i
      if (.s$ = "a")		; valid_nucleus
         result = .i
      endif
      .t1 = Get end point... nucleus_tier .i
   endwhile
endproc


procedure detect_hesitations .t1a .t2a .method$
# <method>
#	"automatic"	use information from phoneme tier, word tier and prosodic features
#	"annotation"	copy info in hesitation tier
   if (.method$ = "automatic")
   ; Detect hesitation from various sources of information: annotation and prosodic features
   ; - from word tier
   ; - from phoneme tier containing "\oe", "\o/", "9", "@" or "2", with additional conditions on duration, slope and pitch interval
      call interval_from_time nucleiID nucleus_tier .t1a first_interval
      call interval_from_time nucleiID nucleus_tier .t2a last_interval
      for .j from first_interval to last_interval
         select nucleiID
         .label$ = Get label of interval... nucleus_tier .j
         if (.label$ = "a") ; "a" indicates syllabic nucleus
            x1 = Get start point... nucleus_tier .j
            x2 = Get end point... nucleus_tier .j
            xmid = x1 + (x2-x1)/2
            .s$ = Get label of interval... pointer_tier .j
            pj = '.s$'
            hesit = 0		; default = no hesitation
            ;.word$ = ""
            ;if (word_tier_in)
            ;   call interval_from_time segmentationID word_tier_in xmid .i
            ;   .word$ = Get label of interval... word_tier_in .i
            ;endif
            if (phones_available)
               call interval_from_time nucleiID phone_tier xmid .i
               .label$ = Get label of interval... phone_tier .i
               px1 = Get start point... phone_tier .i
               px2 = Get end point... phone_tier .i
               select nucldatID
               intST = Get value... pj intrasyllab
               if (index ("=\o/=\oe=9=2=@=", "='label$'=") and px2-px1 >= 0.3 and intST <= 0 and abs(intST) < 3) 
                  hesit = 1
                  call msg detect_hesitations: hesitation detected at 'xmid:3'
               endif
            endif
            if (hesit)
               call interval_from_time nucleiID hesitation_tier xmid .i
               select nucleiID
               Set interval text... hesitation_tier .i H
            endif
         endif
      endfor
      hesitation_available = 1
   endif	; automatic

   ; Copy info in hesitation tier to column of nucldatID
   select nucldatID
   .nrows = Get number of rows
   for .j to .nrows
      .t1n = Get value... .j nucleusstarttime
      .t2n = Get value... .j nucleusendtime
      .tmid = .t1n+(.t2n-.t1n)/2
      select nucleiID
      .i = Get interval at time... hesitation_tier .t1n+(.t2n-.t1n)/2
      .label$ = Get label of interval... hesitation_tier .i
      .hesit = 0
      if (.label$ = "H")
         .hesit = 1
      endif
      select nucldatID
      Set value... .j j_hesitation .hesit
   endfor
endproc


procedure process_cprom: .grid
   call tier_number_by_name .grid "^ss$"
   if (result)
      Remove tier... result
   endif
   call tier_number_by_name .grid "phones"
   if (result)	; Remove labels for pauses ("_"), respirations ("*"), etc. "+"
      ; Replace interval text... result 0 0 "[\+\*_\#\%]" "" Regular Expressions
   endif
   call tier_number_by_name .grid "syll"
   .syll_tier = result
   if (.syll_tier)	; Simplify complex labels for pauses, respirations, etc. to "_"
      ; Replace interval text... .syll_tier 0 0 "[\+\*_\#]+" "_" Regular Expressions
   endif
; Create prominence and hesitation tiers from "delivery" tier in TextGrid <.grid>
      call tier_number_by_name .grid "delivery"
      if (result)
         .deliv_tier = result
         .nt = Get number of tiers
         .prom_tier = .nt + 1
         .hes_tier = .nt + 2
         .ni = Get number of intervals... .deliv_tier
         Duplicate tier... .syll_tier .prom_tier prom
         Duplicate tier... .syll_tier .hes_tier hes
         for .j to .ni
            .label$ = Get label of interval... .deliv_tier .j
            if (index (.label$, "P"))
              .s$ = "S"
            elsif (index (.label$, "p"))
              .s$ = "W"
            else
              .s$ = "0"
            endif
            Set interval text... .prom_tier .j '.s$'
            if (index (.label$, "z"))
              .s$ = "H"
            else
              .s$ = ""
            endif
            Set interval text... .hes_tier .j '.s$'
         endfor
      endif
endproc


procedure lpa_sampa (.grid, .tiername$)
# Convert LPA (Limsi phonetic alphabet) to SAMPA
   call tier_number_by_name .grid "'.tiername$'"
   .syll_tier = result
   if (result)
         Replace interval text... result 0 0 "\." "_" Regular Expressions
         Replace interval text... result 0 0 "\*" "" Regular Expressions
         Replace interval text... result 0 0 "H" "" Regular Expressions
         Replace interval text... result 0 0 "@" "2" Regular Expressions
         Replace interval text... result 0 0 "x" "@" Regular Expressions
         Replace interval text... result 0 0 "h" "H" Regular Expressions
         Replace interval text... result 0 0 "I" "e~" Regular Expressions
         Replace interval text... result 0 0 "O" "o~" Regular Expressions
         Replace interval text... result 0 0 "A" "a~" Regular Expressions
         Replace interval text... result 0 0 "c" "O" Regular Expressions
         Replace interval text... result 0 0 "X" "9" Regular Expressions
         Replace interval text... result 0 0 "r" "R" Regular Expressions
         Replace interval text... result 0 0 "N" "J" Regular Expressions
   endif      
endproc


procedure calculate_pitchrange_from_files .cmda$
   call option_get word file= inputfname "" '.cmda$'
   call files_get_regex "filelistID" 'inputfname$'
   select filelistID
   .nrofFiles = Get number of strings
   if (.nrofFiles == 0)
      call msg No input files found for <'inputfname$'>
   endif
   for .iFile to .nrofFiles
      select filelistID
      .fname$ = Get string... .iFile
      .tmpID = Read from file: .fname$
      .nrows = Get number of rows
      if (.iFile = 1)
         .table = .tmpID
      else
         selectObject: .table, .tmpID
         .appended = Append
         .nrows = Get number of rows
         removeObject: .tmpID, .table
         .table = .appended
      endif
   endfor
   removeObject: filelistID

   select .table
   ; remove rows with f0 discontinuity
   ; .col = Get column index: "f0_discont"
   .row = .nrows
   while (.row > 0)
      .f0_discont = Get value: .row, "f0_discont"
      if (.f0_discont)
          Remove row: .row
      endif
      .row -= 1
   endwhile
   .nrows = Get number of rows
   .tmptableID = Create Table without column names: "pitchvalues", .nrows*2, 4
   Set column label (index): 1, "pitch_Hz"
   Set column label (index): 2, "pitch_ST"
   Set column label (index): 3, "starttime"
   Set column label (index): 4, "f0_Hz"
   speaker_j = 1
   speaker_nnucl [speaker_j] = 0
      .n = 0
      for .j to .nrows
         select .table
         .id = Get value: .j, "speaker_id"
         if (.id == speaker_j)
            speaker_nnucl [speaker_j] += 1
            .t1 = Get value: .j, "nucl_t1"
            .t2 = Get value: .j, "nucl_t2"
            .vlo = Get value: .j, "lopitch"
            .vhi = Get value: .j, "hipitch"
            .f0_hi = Get value: .j, "f0_max"
            .f0_lo = Get value: .j, "f0_min"
            select .tmptableID
            .n += 1
            Set numeric value: .n, "pitch_Hz", .vlo
            Set numeric value: .n, "pitch_ST", hertzToSemitones (.vlo) - hertzToSemitones(1)
            Set numeric value: .n, "starttime", .t1
            Set numeric value: .n, "f0_Hz", .f0_lo
            .n += 1
            Set numeric value: .n, "pitch_Hz", .vhi
            Set numeric value: .n, "pitch_ST", hertzToSemitones (.vhi) - hertzToSemitones(1)
            Set numeric value: .n, "starttime", .t1
            Set numeric value: .n, "f0_Hz", .f0_hi
         endif
      endfor
      if (.n = 0)		; no valid points are available for this speaker, use default values
         call msg Warning: pitchrange measurement: no pitch data for speaker 'speaker_j'  
         mean = 150
         median = 150
         bottom = 150
         top    = 150
         stdev  = 0
         mean_of_ST = 0
         stdev_of_ST = 0
      else
         if (.n < 200)
            call msg Warning: too few syllables for robust pitchrange detection of speaker 'speaker_j'  
         endif
         select .tmptableID
         medianST = Get quantile: "pitch_ST", 0.5
         .row = .n
         while (.row > 0)
            .v = Get value: .row, "pitch_ST"
            if (abs(.v - medianST) > 18)			; discard manifest errors
               Remove row: .row
            endif
            .row -= 1
         endwhile
         .rows = Get number of rows
         mean = Get mean: "pitch_Hz"
         median = Get quantile: "pitch_Hz", 0.5
         bottom = Get quantile: "pitch_Hz", 0.02
         top    = Get quantile: "pitch_Hz", 0.98
         stdev  = Get standard deviation: "pitch_Hz"
         mean_of_ST = Get mean: "pitch_ST"
         stdev_of_ST = Get standard deviation: "pitch_ST"
         mean_rf0 = Get mean: "f0_Hz"
         p50_rf0 = Get quantile: "f0_Hz", 0.5
         p02_rf0 = Get quantile: "f0_Hz", 0.02
         p98_rf0 = Get quantile: "f0_Hz", 0.98
      endif
      if (stdev == undefined)		; when insufficient data
         stdev = 0
      endif
      if (stdev_of_ST == undefined)	; when insufficient data
         stdev_of_ST = 0
      endif
      meanST   = hertzToSemitones (mean) - hertzToSemitones(1)
      medianST = hertzToSemitones (median) - hertzToSemitones(1)
      bottomST = hertzToSemitones (bottom) - hertzToSemitones(1)
      topST    = hertzToSemitones (top) - hertzToSemitones(1)
      removeObject: .tmptableID, .table
      .n = .rows/2			; nrof nuclei used 
      ; label$ = speaker_label'speaker_j'$
      range = topST - bottomST
      upper_range = 12 * log2 (top/median)
      lower_range = range - upper_range
      speaker_range_'speaker_j'$ = 
      ... "BOTTOM_Hz='bottom:0' BOTTOM_ST='bottomST:1' MEDIAN_Hz='median:0' MEDIAN_ST='medianST:1' " +
      ... "MEAN_Hz='mean:0' MEAN_ST='meanST:1' STDEV_HZ='stdev:2' MEAN_OF_ST='mean_of_ST:2' STDEV_OF_ST='stdev_of_ST:3' " +
      ... "TOP_Hz='top:0' TOP_ST='topST:1' " +
      ... "RANGE_ST='range:1' UPPER_RANGE_ST='upper_range:1' LOWER_RANGE_ST='lower_range:1' NROFNUCL='.n' " +
      ... "RF0_P02_Hz='p02_rf0:0' RF0_P98_Hz='p98_rf0:0' RF0_P50_Hz='p50_rf0:0' RF0_MEAN_Hz='mean_rf0:0'"
      s$ = speaker_range_'speaker_j'$ 
   call msg 's$'
   call msg Batch command calculate_pitchrange_from_files: Ready
endproc

